local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"
local _dbg = require "coaf.debug"

local dset = {}
--------------------------------------
local function API_add(red, args)
	if args and args.name then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r, done, msg = proxy.AddPublishPoint(args.name)
			proxy:Destroy()
			if r and done then
				MSG.OK()
			elseif not r then
				-- MSG.ERROR( "添加发布点失败：" .. (done or "未知错误") )
				MSG.ERROR("addPublishPointError")
			else
				-- MSG.ERROR( "添加发布点失败：" .. (msg or "未知错误" ) )
				MSG.ERROR("addPublishPointError")
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
local function API_list(red, args)
	local proxy = CO.GetProxy( "/streamer", "mediaserver" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, ret = proxy.GetPublishPoints()
		proxy:Destroy()
		if r and ret then
			MSG.OK( "", {
				data = ret
			})
		else
			-- MSG.ERROR( "获取发布点列表失败：" .. (ret or "未知错误" ) )
			MSG.ERROR("getPublishPointsError")
		end
	end
	return true
end
--------------------------------------
local function API_modify(red, args)
	if args and args.id and args.name then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r,done = proxy.RenamePublishPoint( args.id, args.name )
			proxy:Destroy()
			if r and done then
				MSG.OK()
			else
				-- MSG.ERROR( "重命名发布点失败：" .. (done or "未知错误") )
				MSG.ERROR("renamePublishPointError")
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
local function API_status(red, args)
	if args and args.id then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r, ret,err = proxy.GetPublishPointStatus(args.id)
			proxy:Destroy()
			if r and ret then
				MSG.OK( "", {
					data = {
						ret
					}
				})
			else
				-- MSG.ERROR( "无法获取错误发布点状态：" .. (err or "未知错误") )
				MSG.ERROR("getPublishPointStatusError")
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
local function API_remove(red, args)
	if args and args.id then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r,done = proxy.RemovePublishPoint(args.id)
			proxy:Destroy()
			if r and done then
				MSG.OK()
			else
				-- MSG.ERROR( "删除发布点失败：" .. (done or "未知错误") )
				MSG.ERROR("delPublishPointError")
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
local function API_reset(red, args)
	if args and args.id then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r,done = proxy.ResetPublishPoint(args.id)
			proxy:Destroy()
			if r and done then
				MSG.OK()
			else
				-- MSG.ERROR( "重置发布点失败：" .. (done or "未知错误") )
				MSG.ERROR("resetPublishPointError")
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
local function API_add_stream(red, args)
    args.id = "1"
    args.head = nil
	if args and args.id and args.type then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r,info = proxy.GetPublishPoints(args.id)
			if not r or not info then
				proxy:Destroy()
				MSG.ERROR( "invalidPointError" )
			else
				proxy:ChangeObject( info.objPath )
				local done
				local sid
				r,sid,info = proxy:CreateStream( args.type )
				if r and sid then
					-----直接设置 相关参数 
					if args.enable then --Cast it to boolean
						if tonumber(args.enable) == 1 then
							args.enable = true
						else
							args.enable = false
						end
					end
					local r, info,err = proxy:SetStreamParameters( tonumber(sid), args )
					proxy:Destroy()
					if r and info then
						MSG.OK()
					else
						-- MSG.ERROR( "无法配置媒体服务参数：" .. (info or "未知") )
						MSG.ERROR("setStreamOptError")
					end
				else
					-- MSG.ERROR( "不能创建流服务：" .. (info or "未知错误") )
					MSG.ERROR(info or "createStreamError")
					proxy:Destroy()
				end
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
local function API_delete_stream(red, args)
    args.head = nil
    args.id = "1"
	if args and args.id and args.stream_id then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r,info = proxy.GetPublishPoints(args.id)
			if not r or not info then
				proxy:Destroy()
				MSG.ERROR( "invalidPointError" )
			else
				proxy:ChangeObject( info.objPath )
				r,info = proxy:RemoveStream( tonumber(args.stream_id) )
				proxy:Destroy()
				if r and info then
					MSG.OK()
				else
					-- MSG.ERROR( "删除流服务失败：" .. (info or "未知错误") )
					MSG.ERROR("delStreamError")
				end
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
local function API_list_stream(red, args)
    args.head = nil
    args.id = "1"
	if args and args.id then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			if args.id =="" then
				MSG.OK( "", { data={} } )                                             
				return true        
			end 

			local r,info = proxy.GetPublishPoints(args.id)
			if not r or not info then
				proxy:Destroy()
				MSG.ERROR( "invalidPointError" )
			else
				proxy:ChangeObject( info.objPath )
				local r, streams = proxy:GetStreams()
				proxy:Destroy()
				if not r then
					-- MSG.ERROR( "获取流失败：" .. (streams or "未知错误") )
					MSG.ERROR("getStreamError")
				else
					if not streams then
						MSG.OK( "", { data={} } )
					else
						local data = {}
						local i, addr
						for i=1, #streams do
							local addr = streams[i].address or ""
							local pfx,tok,sfx = addr:match("^(.*)(%%HOST%%)(.*)$")
							if pfx then
								addr = pfx .. ngx.var.server_addr .. (sfx or "")
							end

							table.insert( data, {
								stream_id = streams[i].id,
								enable = streams[i].enable and 1 or 0,
								type = streams[i].type,
								status = streams[i].status,
								name = streams[i].name,
								address = addr,
								error_reason = streams[i].error_reason
							} )
						end

						MSG.OK( "", { data = data } )
					end
				end
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
local function API_get_stream_config(red, args)
    args.id = "1"
    args.head = nil
	if args and args.id and args.stream_id then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r,info = proxy.GetPublishPoints(args.id)
			if not r or not info then
				proxy:Destroy()
				MSG.ERROR( "invalidPointError" )
			else
				proxy:ChangeObject( info.objPath )
				r, info = proxy:GetStreamParameters( tonumber(args.stream_id) )
				proxy:Destroy()
				if not r or not info then
					MSG.ERROR( "getStreamOptError" )
				else
					info["enable"] = info["enable"] and 1 or 0
					MSG.OK( "", {
						data = { info }
					})
				end
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
local function API_set_stream_config(red, args)
    args.head = nil
    args.id = "1"
	if args and args.id and args.stream_id then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r,info = proxy.GetPublishPoints(args.id)
			if not r or not info then
				proxy:Destroy()
				MSG.ERROR( "invalidPointError" )
			else
				proxy:ChangeObject( info.objPath )
				local sid = tonumber(args.stream_id)
				args.id = nil
				args.stream_id = nil	--Clear these items.
				if args.enable then --Cast it to boolean
					if tonumber(args.enable) == 1 then
						args.enable = true
					else
						args.enable = false
					end
				end
				r, info,err = proxy:SetStreamParameters( sid, args )
				proxy:Destroy()
				if r and info then
					MSG.OK()
				else
					-- MSG.ERROR( " " .. (info or "未知") )
					MSG.ERROR(err or "setStreamOptError")
				end
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
local function API_enable_stream(red, args)
	return API_set_stream_config(red, args)
end
--------------------------------------
local function API_get_source(red, args)
	local current_stream
	local proxy
	if args and args.id then
		proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			_dbg.Error( "STREAMER", "Fail to create proxy to codec:/streamer" )
		else
			local r,info,err = proxy.GetPublishPointStatus(args.id)
			proxy:Destroy()
			if r and info then
				current_stream = info.current_stream
			end
		end
	end

	proxy = CO.GetProxy( "/streams", "mediaserver" )
	if not proxy then
		MSG.ERROR( "mediaserverError" )
		return true
	else
		local r, streams = proxy.GetStreams({"local","manual","remote","output"}, "with-sub")
		proxy:Destroy()

		if not r or not streams then
			-- MSG.ERROR( "无法获取媒体源列表" .. (streams or "ERROR") )
			MSG.ERROR("getStreamsError")
			return true
		end

		local result = {
		}

		local i, item
		-- status = streams[i].type == "manual" and "off" or "on",
		for i=1, #streams do
			item = {
				id = streams[i].id,
				name = streams[i].name,
				current = (streams[i].id == current_stream and 1 or 0),
				type = streams[i].type,
				sourceType = streams[i].sourceType,
				origin = streams[i].origin,
				src = streams[i].address,
				status =  streams[i].status or "off",
				resolution = "分辨率未知",
				--name_position = "left_top", --TODO
				--time_position = "left_top",
				manual_url = streams[i].manual_url or "",
				--buffer = streams[i].buffer_option or "live:200:200:500:0:0",
				bitrate = 0
			}

			if streams[i].stream_info then
				if streams[i].stream_info.live == "connected" then
					if streams[i].stream_info.stream_start and streams[i].stream_info.output_start then
						item.status = "view_pull"
					elseif streams[i].stream_info.stream_start then
						item.status = "pull"
					elseif streams[i].stream_info.output_start then
						item.status = "view"
					else
						item.status = "on"
					end
				else
					if (streams[i].type == "manual" and streams[i].stream_info.live and streams[i].stream_info.live ~= "") or (streams[i].stream_info.live ~= "closed" and streams[i].stream_info.live ~= "") then
						item.status = streams[i].stream_info.live
					end
				end

				item.resolution = streams[i].stream_info.resolution or "分辨率未知"
				item.bitrate = streams[i].stream_info.bitrate or 0
			end

			table.insert( result, item )
		end

		MSG.OK( "", {
			data = result
		})

		return true
	end
end
--------------------------------------
local function API_select_source(red, args)
	if args and args.id and args.source_id then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r,info = proxy.GetPublishPoints(args.id)
			if not r or not info then
				proxy:Destroy()
				MSG.ERROR( "invalidPointError" )
			else
				proxy:ChangeObject( info.objPath )
				r,info = proxy:SelectSource( args.source_id )
				proxy:Destroy()
				if r and info then
					MSG.OK()
				else
					-- MSG.ERROR( "选择媒体源失败：" .. (info or "未知错误") )
					MSG.ERROR("selectSourceError")
				end
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
dset.APIS = dset.APIS or {}

dset.APIS["add"] = API_add
dset.APIS["list"] = API_list
dset.APIS["modify"] = API_modify
dset.APIS["remove"] = API_remove
dset.APIS["status"] = API_status
dset.APIS["reset"] = API_reset
dset.APIS["add_stream"] = API_add_stream
dset.APIS["delete_stream"] = API_delete_stream
dset.APIS["enable_stream"] = API_enable_stream
dset.APIS["get"] = API_list_stream
dset.APIS["get_stream_config"] = API_get_stream_config
dset.APIS["set_stream_config"] = API_set_stream_config
dset.APIS["get_source"] = API_get_source
dset.APIS["select_source"] = API_select_source

--------------------------------------
return dset

