local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"
local _dbg = require "coaf.debug"

local dset = {}
--------------------------------------
local function API_get(red, args)
	local current_stream
	local proxy
	proxy = CO.GetProxy( "/streamer", "mediaserver" )
	if not proxy then
		_dbg.Error( "STREAMER", "Fail to create proxy to codec:/streamer" )
	else
		local r,info,err = proxy.GetPublishPointStatus("1")
		proxy:Destroy()
		if r and info then
			current_stream = info.current_stream
		end
	end

	local proxy = CO.GetProxy( "/streams", "mediaserver" )
	if not proxy then
		MSG.ERROR( "mediaserverError" )
		return true
	else
		local r, streams = proxy.GetStreams({"local","manual","remote"})
		proxy:Destroy()

		if not r or not streams then
			MSG.ERROR( "getStreamsError" )
			-- MSG.ERROR( "无法获取媒体源列表" .. (streams or "ERROR") )
			return true
		end

		local result = {
		}

		local i, item
		for i=1, #streams do
			if not args or not args.id or args.id == streams[i].id then
				if not streams[i].sourceType or streams[i].sourceType == "main" then
					local originIp,imgUrl
					if streams[i].origin =="remote" then
						local num = string.find(streams[i].imgUrl,"%/")
						originIp= string.sub(streams[i].imgUrl,1,num-1)
						imgUrl = string.sub(streams[i].imgUrl,num)
					else
						imgUrl = streams[i].imgUrl
					end
					
					--只返回主码流的动态数据
					item = {
						id = streams[i].id,
						name = streams[i].name,
						type = streams[i].type,
						src = streams[i].address or streams[i].manual_url,
						enable = (streams[i].id == current_stream and 1 or 0),
						status = streams[i].origin == "local" and "on" or "off",
						resolution = "分辨率未知",
						name_position = "left_top", --TODO
						time_position = "left_top",
						icons = streams[i].icons,
						manual_url = streams[i].manual_url or "",
						buffer = streams[i].buffer_option or "live:200:200:500:0:0",
						bitrate = 0,
						origin= streams[i].origin,
						address  = streams[i].address or  streams[i].manual_url,
						imgUrl = imgUrl,
						originIp = originIp
					}
					
					if streams[i].stream_info then
						if streams[i].stream_info.live == "connected" then
							if streams[i].stream_info.stream_start and streams[i].stream_info.output_start then
								item.status = "view_pull"
							elseif streams[i].stream_info.stream_start then
								item.status = "pull"
							elseif streams[i].stream_info.output_start then
								item.status = "view"
							else
								item.status = "on"
							end
						else
							if (streams[i].type == "manual" and streams[i].stream_info.live and streams[i].stream_info.live ~= "") or (streams[i].stream_info.live ~= "closed" and streams[i].stream_info.live ~= "") then
								item.status = streams[i].stream_info.live
							end
						end

						item.resolution = streams[i].stream_info.resolution or "分辨率未知"
						item.bitrate = streams[i].stream_info.bitrate or 0
					end

					if streams[i].srt_status then
						item.srtStatus = streams[i].srt_status
					end

					if streams[i].rtmp_server_status then
						item.rtmp_server_status =streams[i].rtmp_server_status
					end
				end
				table.insert( result, item )
			end
		end
		local none
		if not current_stream or current_stream == "" then
			none = true
		else 
			none = false
		end

		MSG.OK( "", {
			data = result,
			none = none

		})

		return true
	end
end


local function API_get_source(red, args)
	local current_stream
	local proxy
	proxy = CO.GetProxy( "/streamer", "mediaserver" )
	if not proxy then
		_dbg.Error( "STREAMER", "Fail to create proxy to codec:/streamer" )
	else
		local r,info,err = proxy.GetPublishPointStatus("1")
		proxy:Destroy()
		if r and info then
			current_stream = info.current_stream
		end
	end

	proxy = CO.GetProxy( "/streams", "mediaserver" )
	if not proxy then
		MSG.ERROR( "mediaserverError" )
		return true
	else
		local r, streams = proxy.GetStreams({"local","manual","remote","output"})
		proxy:Destroy()

		if not r or not streams then
			-- MSG.ERROR( "无法获取媒体源列表" .. (streams or "ERROR") )
			MSG.ERROR("getStreamsError")
			return true
		end

		local result = {
		}

		local i, item
		-- status = streams[i].type == "manual" and "off" or "on",
		for i=1, #streams do
			item = {
				id = streams[i].id,
				name = streams[i].name,
				enable = (streams[i].id == current_stream and 1 or 0),
				type = streams[i].type,
				sourceType = streams[i].sourceType,
				origin = streams[i].origin,
				src = streams[i].address,
				status =  streams[i].status or "off",
				resolution = "分辨率未知",
				--name_position = "left_top", --TODO
				--time_position = "left_top",
				manual_url = streams[i].manual_url or "",
				--buffer = streams[i].buffer_option or "live:200:200:500:0:0",
				bitrate = 0
			}

			if streams[i].stream_info then
				if streams[i].stream_info.live == "connected" then
					if streams[i].stream_info.stream_start and streams[i].stream_info.output_start then
						item.status = "view_pull"
					elseif streams[i].stream_info.stream_start then
						item.status = "pull"
					elseif streams[i].stream_info.output_start then
						item.status = "view"
					else
						item.status = "on"
					end
				else
					if (streams[i].type == "manual" and streams[i].stream_info.live and streams[i].stream_info.live ~= "") or (streams[i].stream_info.live ~= "closed" and streams[i].stream_info.live ~= "") then
						item.status = streams[i].stream_info.live
					end
				end

				item.resolution = streams[i].stream_info.resolution or "分辨率未知"
				item.bitrate = streams[i].stream_info.bitrate or 0
			end

			table.insert( result, item )
		end

		MSG.OK( "", {
			data = result,
		})

		return true
	end
end


--------------------------------------
--简单的URL检查 必须包含-> protocol://orig_url_tail 文件类型除外
local function CheckURL(url)
	local protocol, orig_url_tail
	if type(url) == "table"  then --file type
		if #(url or {}) == 0 then
			return false
		else
			return true
		end
	end

	protocol, orig_url_tail = url:match( "(%w+)://(.+)$" )
	if not protocol or not orig_url_tail then
		return false
	end

	return true
end

local function CreateOptions( args )
	local options = {}
	if args.type and args.type:lower() == "ndi" then
		options.url = args.manual_url
		options.type = args.type
		options.buffer_option = args.buffer or "live:200:200:500:0:0"
		options.ndi_name =  args.ndi_name or ""
		options.preview_mode =  args.preview_mode or 0	
	elseif args.type and args.type:lower() == "srt" then
		options.type = args.type
		options.url = args.manual_url
		options.address = args.address or nil
		options.listenerPort = args.listenerPort or nil
		options.buffer_option = args.buffer or "live:200:200:500:0:0"
		options.connectionMode =  args.connectionMode or nil
		options.passphrase =  args.passphrase or nil
		options.encryption =  args.encryption or nil
		options.payloadSize =  args.payloadSize or 1316
		options.bandwidth =  args.bandwidth or 1316
		options.latency =  args.latency or 125
		options.connect_speed = (args.connect_speed and tonumber(args.connect_speed)) or 2
		options.showAdvanced = args.showAdvanced or 0
		options.srt_streamid = args.srt_streamid or ""
	elseif args.type and args.type:lower() == "file" then
		--ngx.log(ngx.ERR, string.format("file_list len = %d", #args.file_list))
		for _, v in pairs(args.file_list) do
			v.disk = nil
			v.id = nil
			--v.name = nil
			v.totalDuration = nil
			v.work = nil
		end

		options.type = args.type
		options.buffer_option = args.buffer or "live:200:200:500:0:0"
		options.loop = args.loop or false
		options.file_list = args.file_list
	elseif args.type and args.type:lower() == "rtmp_server" then
		options.type = args.type
		options.url = args.manual_url
		options.buffer_option = args.buffer or "live:200:200:500:0:0"
		options.rtmp_server_port = args.rtmp_server_port or 1935
		options.rtmp_server_enable = args.rtmp_server_enable or false
		options.rtmp_server_security = args.rtmp_server_security or "none"
		options.rtmp_server_session = args.rtmp_server_session
		options.rtmp_server_token = args.rtmp_server_token
		options.rtmp_server_application = args.rtmp_server_application
		options.username = args.user or nil
		options.password = args.password or nil
		options.connect_speed = (args.connect_speed and tonumber(args.connect_speed)) or 2
	else
		options.type = args.type
		options.url = args.manual_url
		options.buffer_option = args.buffer or "live:200:200:500:0:0"
		options.username = args.user or nil
		options.password = args.password or nil
		options.bind =  args.bind_net or nil
		options.transport = args.trans_mode or nil
		options.rtp_video_port = (args.video_port and tonumber(args.video_port)) or nil
		options.rtp_audio_port = (args.audio_port and tonumber(args.audio_port)) or nil
		options.rtp_video_codec = args.video_codec or nil
		options.rtp_audio_codec = args.audio_codec or nil
		options.connect_speed = (args.connect_speed and tonumber(args.connect_speed)) or 2
		options.imgUrl = args.imgUrl or ""
	end

	return options
end


local function API_modify(red, args)

	if args and args.id and args.name and args.main and args.sub then
		local proxy = CO.GetProxy( "/streams", "mediaserver" )
		if not proxy then
			MSG.ERROR( "mediaserverError" )
			return true
		else
			local r, done 
			if args.id:match("^LOCAL-") or args.id:match("^OUTPUT") then
				--Only change name for local or output stream.
				r, done = proxy.ChangeStreamDetail( args.id, {
					name = args.name
				})
			else
				local is_sub = true
				local main, sub

				if not CheckURL(args.main.file_list or args.main.manual_url) then
					MSG.ERROR( "Invalid main URL！" )
					proxy:Destroy()
					return true
				end

				if not CheckURL(args.sub.file_list or args.sub.manual_url) then
					is_sub = false
				end

				main = CreateOptions(args.main)
				sub = CreateOptions(args.sub)

				r, done = proxy.ChangeStreamDetail( args.id, {
					name = args.name,
					with_sub = is_sub,
					options = main,
					sub_options = sub,
					audio_enables = args.audio_enables or {}
				})		
			end

			if not r or not done then
				MSG.ERROR( "modifyStreamArgError")
				-- MSG.ERROR( "无法修改流参数:" .. (done or "未知错误") )
			else
				MSG.OK()
			end

			proxy:Destroy()
			return true
		end
	elseif args and args.id and args.enable then
		local proxy = CO.GetProxy( "/streamer", "mediaserver" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r,info = proxy.GetPublishPoints("1")
			if not r or not info then
				proxy:Destroy()
				MSG.ERROR( "invalidPointError" )
			else
				proxy:ChangeObject( info.objPath )
				if tonumber(args.enable) == 1 then
					r,info = proxy:SelectSource( args.id)
				else
					r,info = proxy:SelectSource( "","" )
				end
				proxy:Destroy()
				if r and info then
					MSG.OK()
				else
					-- MSG.ERROR( "选择媒体源失败：" .. (info or "未知错误") )
					MSG.ERROR("selectSourceError")
				end
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

--------------------------------------

local function API_add(red, args)
	local proxy = CO.GetProxy( "/streams", "mediaserver" )
	if not proxy then
		MSG.ERROR( "mediaserverError" )
		return true
	end

	local r, done

	if args  and  args.name and  args.main and  args.sub then
		local sub_enable = true
		local islocal = false
		local main_options, sub_options

		if not CheckURL(args.main.file_list or args.main.manual_url) then
			MSG.ERROR( "Invalid main URL！" )
			proxy:Destroy()
			return true
		end

		if not CheckURL(args.sub.file_list or args.sub.manual_url) then
			sub_enable = false
		end

		main_options = CreateOptions(args.main)
		sub_options = CreateOptions(args.sub)
		r, done = proxy.AddNetworkStream( "MANUAL.", args.name, main_options, islocal, sub_enable, sub_options)
	else
		MSG.ERROR( "argumentError" )
		proxy:Destroy()
		return true
	end

	if not r or not done then
		MSG.ERROR( "addNetStreamError" )
	else
		MSG.OK()
	end
	
	proxy:Destroy()
	return true
end

--------------------------------------

local function API_remove(red, args)
	if args and args.id then
		local proxy = CO.GetProxy( "/streams", "mediaserver" )
		if not proxy then
			MSG.ERROR( "mediaserverError" )
			return true
		else
			local r,done = proxy.RemoveNetworkStream( args.id )
			proxy:Destroy()
			if not r or not done then
				MSG.ERROR( "delNetStreamError")
			else
				MSG.OK()
			end
			return true
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end


local function API_getSourceConfig(red, args)
	local proxy = CO.GetProxy( "/streams", "mediaserver" )
	if not proxy then
		MSG.ERROR( "mediaserverError" )
		return true
	else
		local r, config = proxy.GetNetSourceConfig(args.id)
		proxy:Destroy()

		if not r or not config then
			MSG.ERROR( "getSourceConfigError" )
			return true
		end
		
		if config.options then 
			config.options.user = config.options.username or ""
			config.options.username = nil
			config.options.manual_url = config.options.url or ""
			config.options.url = nil
			config.options.buffer = config.options.buffer_option or "live:200:200:500:0:0"
			config.options.buffer_option = nil
			config.options.trans_mode = config.options.transport or ""
			config.options.transport = nil
		end

		if config.sub_options then 
			config.sub_options.user = config.sub_options.username or ""
			config.sub_options.username = nil
			config.sub_options.manual_url = config.sub_options.url or ""
			config.sub_options.url = nil
			config.sub_options.buffer = config.sub_options.buffer_option or "live:200:200:500:0:0"
			config.sub_options.buffer_option = nil
			config.sub_options.trans_mode = config.sub_options.transport or ""
			config.sub_options.transport = nil
		end

		local result = {
			id = args.id,
			name = config.name or "",
			audio_enables = config.audio_enables or false,
			manual_type = config.manual_type or "manual",
			manual = config.manual or true,
			main = config.options or nil,
			sub = config.sub_options or nil
		}

		MSG.OK( "", {
			data = result
		})

		return true
	end
end

-------------------------------------------------------------------------

local function API_search(red, args)
	--TODO:
	MSG.OK( "", {
		data = {
			ONVIF = {
				{
					id = "invalid",
					name = "暂未支持发现Onvif设备<br/>",
					ip = "0.0.0.0",
					type = "IPCAM",
					flag = "ONVIF"
				}
			},
			KILOVIEW = {
				{
					id = "invalid",
					name = "暂未支持发现KILOVIEW设备<br/>",
					ip = "0.0.0.0",
					type = "ENCODER",
					flag = "KILOVIEW"
				}
			},
			["NDI-HX"] = {
				{
					id = "invalid",
					name = "暂未支持发现NDI-HX设备<br/>",
					ip = "0.0.0.0",
					type = "NDI",
					flag = "NDI-HX"
				}
			}
		}
	})
	return true
end


--一键清空所有网络源
local function API_empty(red,args)
	local proxy = CO.GetProxy( "/streams", "mediaserver" ,"local", 10)
	if not proxy then
		MSG.ERROR( "mediaserverError" )
		return true
	else
		local r, done,err = proxy.RemoveAllNetworkStream()
		if not r or not done then
			-- MSG.ERROR( "一键清空所有网络源 失败:",err )
			MSG.ERROR("removeAllNetStreamError")
		else
			MSG.OK("")
		end
		proxy:Destroy()

	end
	return true
end


local function API_test(red,args)
    local headers =ngx.req.get_headers()
    MSG.OK("",{data=headers})
    
    return true

end

local function ConvertDB(num, skip)
	if skip then
		return math.floor((num + 121) / (121 + 6) * 200)
	else
		return math.floor(num / 200 * (121 + 6)) - 121
	end
end


local function API_getRtmpServicePort(red,args)
	local proxy = CO.GetProxy("/rtmp_service", "rtmp_service")
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, done = proxy.GetServerPort()
		proxy:Destroy()
		if not r or not done then
			MSG.ERROR( "getServerPort" )
		else
			MSG.OK("",{data= {rtmp_server_port = done}})
		end
		return true
	end
end

local function API_setRtmpServicePort(red,args)
	if not args.rtmp_server_port  then
		MSG.ERROR( "argumentError" )
		return true
	end

	local proxy = CO.GetProxy("/rtmp_service", "rtmp_service")
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, done = proxy.UpdateServerPort(args.rtmp_server_port)
		proxy:Destroy()
		if not r or not done then
			MSG.ERROR( "setServerPort" )
			return true
		--else
		--	MSG.OK("")
		end
	end

	local proxy = CO.GetProxy( "/streams", "mediaserver" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, done = proxy.UpdateRtmpSourcePort(args.rtmp_server_port)
		proxy:Destroy()
		if not r or not done then
			MSG.ERROR( "setServerPort" )
		else
			MSG.OK("")
		end
	end

	return true		
end


local function API_status(red,args)
	local proxy = CO.GetProxy("/streams", "mediaserver")
	if not proxy then
		MSG.ERROR( "mediaserverError" )
		return true
	else
		local r, done = proxy.GetStreamStatus()
		proxy:Destroy()
		if not r or not done then
			MSG.ERROR( "getServerPort" )
		else
			MSG.OK("",{data= done})
		end
		return true
	end
end


--------------------------------------
dset.APIS = dset.APIS or {}

dset.APIS["get"] = API_get
dset.APIS["add"] = API_add
dset.APIS["remove"] = API_remove
dset.APIS["modify"] = API_modify
dset.APIS["search"] = API_search
dset.APIS["getSourceConfig"] = API_getSourceConfig

dset.APIS["empty"] = API_empty
dset.APIS["test"] = API_test
dset.APIS["status"] = API_status

dset.APIS["getRtmpServicePort"] = API_getRtmpServicePort
dset.APIS["setRtmpServicePort"] = API_setRtmpServicePort
--------------------------------------
return dset

