local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"
local _dbg = require "coaf.debug"

local dset = {}
--------------------------------------
--[[
api ： 
    add_webrtc，
参数 ：
   stream_id: 数据流的ID号
   ip: 设备的ip地址
   mode：webrtc会话传输数据时默认采用的模式，full:全帧率、key:关键帧；(非必传，默认key)
   port: webrtc会话使用的socket端口(非必传)
   video_type: 视频数据传输模式，recvonly/sendonly/sendrecv；（非必传, 默认 sendonly）
   audio_type: 视频数据传输模式，recvonly/sendonly/sendrecv；（非必传, 默认 sendonly）
--]]
local function API_add_webrtc(red, args)
  if args and args.stream_id and args.ip then 
    local proxy = CO.GetProxy( "/webrtc", "mediaserver" )
    if not proxy then
      MSG.ERROR( "无法连接系统服务" )
      return true
    else 
      local r, rtc = proxy.CreateWebRTCServer(args.stream_id, args.ip, args.mode, args.port, args.video_type, args.audio_type)
      proxy:Destroy()
      if r and rtc then 
        MSG.OK( "", {
          data = rtc
        })
      else
        MSG.ERROR( "无法连接系统服务" )
      end
    end
  else
    MSG.ERROR( "参数错误" )
  end
  return true
end

--[[
api ： 
    set_webrtc，
参数 ：
   mode: key, full (key表示只预览关键帧，full表示全帧率预览)
   id:[40643, 40644] （id的数组集合，通过add_webrtc 接口获得）
--]]
local function API_set_webrtc(red, args)
  if args and args.mode and args.id then 
    local proxy = CO.GetProxy( "/webrtc", "mediaserver" )
    if not proxy then
      MSG.ERROR( "无法连接系统服务" )
      return true
    else 
      local r, rtc = proxy.SetWebRTCVideoSendMode(args.mode, args.id)
      proxy:Destroy()
      if r and rtc then 
        MSG.OK( "" )
      else
        MSG.ERROR( "设置错误" )
      end
    end
  else
    MSG.ERROR( "参数错误" )
  end
  return true
end
--------------------------------------
local function API_delete_webrtc(red, args)
	if args and args.stream_id then 
		local proxy = CO.GetProxy( "/webrtc", "mediaserver" )
		if not proxy then
			MSG.ERROR( "无法连接系统服务" )
			return true
		else 
			proxy.RemoveWebRTCSession(args.stream_id)
			proxy:Destroy()
		end
	else
		MSG.ERROR( "无效的提交参数" )
	end
	return true
end
--------------------------------------


dset.APIS = dset.APIS or {}

dset.APIS["add_webrtc"] = API_add_webrtc
dset.APIS["set_webrtc"] = API_set_webrtc
--------------------------------------
return dset

