local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local periphery = require "periphery"
local GPIO = periphery.GPIO

local _M = {
	pins = {}
}

----------------------------------------------------

local pin_meta = {}

function pin_meta:read()
	if self.drv then
		if not self.revert then
			return self.drv:read()
		else
			return not self.drv:read()
		end
	else
		return false
	end
end

function pin_meta:write(val)
	self.value = val
	self:update(true)
end

function pin_meta:update(no_setting)
	if not no_setting then
		self.drv = GPIO(self.pin, self.direction)
	end
	if self.drv and self.direction == "out" then
		local val = self.value
		if self.revert then
			val = not val
		end
		self.drv:write(val and true or false)
	end
end

pin_meta.__index = pin_meta

----------------------------------------------------

function _M.Pin(pin, direction, revert)
	pin = tonumber(pin)
	if not pin then
		_dbg.Error("GPIO", "Invalid pin number")
		return nil
	end
	
	revert = revert and true or false

	direction = direction or "out"
	direction = direction:lower()
	--local orig_direction = direction
	
	local init_val
	if direction == "high" then
		direction = "out"
		init_val = true
	elseif direction == "low" then
		direction = "out"
		init_val = false
	elseif direction ~= "out" and direction ~= "in" then
		_dbg.Error("GPIO", "Bad pin direction of '", direction, "', should be in/out/low/high." )
		return nil
	end

	local pin_obj = _M.pins[pin]
	if pin_obj then
		if pin_obj.direction ~= direction or (init_val ~= nil and pin_obj.value ~= init_val) or (revert ~= pin_obj.revert) then
			_dbg.Warn("GPIO", "Changed the pin settings to dir=", direction, ",init=", init_val, ",revert=", revert)
			pin_obj.direction = direction
			if init_val ~= nil then
				pin_obj.value = init_val
			end
			pin_obj.revert = revert
			pin_obj:update()
		end
		return pin_obj
	end

	pin_obj = {
		pin = pin,
		direction = direction,
		value = init_val,
		revert = revert
	}

	setmetatable(pin_obj, pin_meta)
	_M.pins[pin] = pin_obj
	pin_obj:update()

	return pin_obj
end

return _M
