local I2C_GPIO = require "chips.tca6424a"
local TEMPERATURE = require "chips.lm75adp"
local FAN = require "chips.fan"
local GPIO = require "chips.gpio"

--Use TCA6424A I2C externsion GPIO.
local iochip1 = I2C_GPIO.Chip("/dev/i2c-0", 0x22)
local iochip2 = I2C_GPIO.Chip("/dev/i2c-0", 0x23)

iochip1:Reset()
iochip2:Reset()

local temp_chip = TEMPERATURE.Chip("/dev/i2c-0", 0x48)
local fan = FAN.GPIOFan( temp_chip, 960, false )

local _M = {
	IO_STEP_TIME = 0.1, --second
	RESTORE_TIME = 3000,   --ms.
	DEFAULT_BLINK_STEPS = {false, false, false, false, false, true, true, true, true, true},

	----------------------------
	--On board LED.
	----------------------------
	board_led = {
		drv = GPIO.Pin(906,"out"),
		state = "blink",
		--You can specify any blink steps for special case.
		--blink_steps = { false, false, false, true, true, true },
	},
	
	----------------------------
	--Temperature sensors.
	----------------------------
	temperature = {
		[1] = { drv = temp_chip }
	},

	----------------------------
	--Fan controllers.
	----------------------------
	fan = {
		[1] = {
			drv = fan,

			--Speed table:
			--[[
			{
				--Lower than, disable.
				{temp = 40, speed = 10}, -- >= 40C, fan speed = 10%
				{temp = 42, speed = 20}, -- >= 42C, fan speed = 20%
				{temp = 45, speed = 30, threshold = true},  -- threshold temperature.
															-- If >= 'threshold', start the fan.
				...
				{temp = 58, speed = 100},
				--Upper than, fan speed = 100%
			}
			--]]
			speed_table = {
				{temp = 45, speed = 10},
				{temp = 46, speed = 20},
				{temp = 47, speed = 30},
				{temp = 48, speed = 50, threshold = true},
				{temp = 50, speed = 60},
				{temp = 52, speed = 70},
				{temp = 55, speed = 80},
				{temp = 58, speed = 90},
				{temp = 60, speed = 100}
			}
		}
	},

	----------------------------
	--Tally LEDs.
	----------------------------
	tally_leds = {
		--You can specify any blink steps for special case.
		--blink_steps = { false, false, false, true, true, true }
		[1] = { red = { drv = GPIO.Pin(965, "out", true) }, green = { drv = GPIO.Pin(966, "out", true) } },

		[2] = { red = { drv = iochip1:Pin(1,"out") }, green = { drv = iochip1:Pin(17, "out") } },
		[3] = { red = { drv = iochip1:Pin(2,"out") }, green = { drv = iochip1:Pin(18, "out") } },
		[4] = { red = { drv = iochip1:Pin(3,"out") }, green = { drv = iochip1:Pin(19, "out") } },
		[5] = { red = { drv = iochip1:Pin(4,"out") }, green = { drv = iochip1:Pin(20, "out") } },
		[6] = { red = { drv = iochip1:Pin(5,"out") }, green = { drv = iochip1:Pin(21, "out") } },
		[7] = { red = { drv = iochip1:Pin(6,"out") }, green = { drv = iochip1:Pin(22, "out") } },
		[8] = { red = { drv = iochip1:Pin(7,"out") }, green = { drv = iochip1:Pin(23, "out") } },
		[9] = { red = { drv = iochip1:Pin(8,"out") }, green = { drv = iochip1:Pin(24, "out") } },
		[10] = { red = { drv = iochip1:Pin(9,"out") }, green = { drv = iochip1:Pin(13, "out") } },

		[11] = { red = { drv = iochip2:Pin(9,"out") }, green = { drv = iochip2:Pin(13, "out") } },

		[12] = { red = { drv = iochip2:Pin(1,"out") }, green = { drv = iochip2:Pin(17, "out") } },
		[13] = { red = { drv = iochip2:Pin(2,"out") }, green = { drv = iochip2:Pin(18, "out") } },
		[14] = { red = { drv = iochip2:Pin(3,"out") }, green = { drv = iochip2:Pin(19, "out") } },
		[15] = { red = { drv = iochip2:Pin(4,"out") }, green = { drv = iochip2:Pin(20, "out") } },
		[16] = { red = { drv = iochip2:Pin(5,"out") }, green = { drv = iochip2:Pin(21, "out") } },
		[17] = { red = { drv = iochip2:Pin(6,"out") }, green = { drv = iochip2:Pin(22, "out") } },
		[18] = { red = { drv = iochip2:Pin(7,"out") }, green = { drv = iochip2:Pin(23, "out") } },
		[19] = { red = { drv = iochip2:Pin(8,"out") }, green = { drv = iochip2:Pin(24, "out") } },
	},

	----------------------------
	--Reset(restore factory) button.
	----------------------------
	reset_button = {
		drv = GPIO.Pin(952, "in") --TODO?
	},

	----------------------------
	-- On demo LED animation.
	----------------------------
	on_demo = {
		on = "startup",
		stage_index = 1,
		stages = {
		--See below of this file:
		--	startup = {...},
		--	restore = {...},
		--	reset = {...},
		--	...
		}
	},

	----------------------------
	-- Signal indicator
	----------------------------
	signal = {
		--You can specify any blink steps for special case.
		--blink_steps = { false, false, false, true, true, true }

		-- drv = { red = GPIO.Pin(...),   --Specify a GPIO pin for red,
		--         green = GPIO.Pin(...)  --Specify a GPIO pin for green
		--         -- Or,
		--         any = GPIO.Pin(...)    --Any color (only one color case)
		-- },
		
		tally_light_id = {6, 15},       --Reuse the tally light, or
		drv = {
			any = GPIO.Pin(964, "out"), --Specify a GPIO pin.
		},

		red = "off",
		green = "off"
	}
}

----------------------------------------------------------------

local ON = false
local OFF = true
local HOLD = "HOLD"

local GROUP_1_START = 1
local GROUP_1_END = 10

local GROUP_2_START = 11
local GROUP_2_END = 19

-- The LED animation when startup.
local demo = {}

--Round 1, green OFF red ON.
table.insert(demo, { _M.fan[1].drv, 100 }) --Let FAN 100% run.
for i=GROUP_1_START, GROUP_1_END do
	table.insert(demo, {_M.tally_leds[i].red.drv, ON})
	table.insert(demo, {_M.tally_leds[i].green.drv, OFF})
	local x = i + GROUP_2_START - 1
	if x <= GROUP_2_END then
		table.insert(demo, {_M.tally_leds[x].red.drv, ON})
		table.insert(demo, {_M.tally_leds[x].green.drv, OFF})
	end
	--Hold 1 step.
	table.insert(demo, {HOLD, 1})
end

--Round 2, green ON red OFF
table.insert(demo, { _M.fan[1].drv, 0 }) --Let FAN stop.
for i=GROUP_1_START, GROUP_1_END do
	table.insert(demo, {_M.tally_leds[i].red.drv, OFF})
	table.insert(demo, {_M.tally_leds[i].green.drv, ON})
	local x = i + GROUP_2_START - 1
	if x <= GROUP_2_END then
		table.insert(demo, {_M.tally_leds[x].red.drv, OFF})
		table.insert(demo, {_M.tally_leds[x].green.drv, ON})
	end
	--Hold 1 step.
	table.insert(demo, {HOLD, 1})
end

--Round 3, all OFF step by step
for i=GROUP_1_START, GROUP_1_END do
	table.insert(demo, {_M.tally_leds[i].red.drv, OFF})
	table.insert(demo, {_M.tally_leds[i].green.drv, OFF})
	local x = i + GROUP_2_START - 1
	if x <= GROUP_2_END then
		table.insert(demo, {_M.tally_leds[x].red.drv, OFF})
		table.insert(demo, {_M.tally_leds[x].green.drv, OFF})
	end
	--Hold 1 step.
	table.insert(demo, {HOLD, 1})
end

_M.on_demo.stages["startup"] = demo

-- The LED animation when restore factory settings.
demo = {}
--Round 1, green ON red OFF.
for i=GROUP_1_START, GROUP_1_END do
	table.insert(demo, {_M.tally_leds[i].red.drv, OFF})
	table.insert(demo, {_M.tally_leds[i].green.drv, ON})
	local x = i + GROUP_2_START - 1
	if x <= GROUP_2_END then
		table.insert(demo, {_M.tally_leds[x].red.drv, OFF})
		table.insert(demo, {_M.tally_leds[x].green.drv, ON})
	end
	--Hold 1 step.
	table.insert(demo, {HOLD, 1})
end

--Round 2, green OFF red ON
for i=GROUP_1_START, GROUP_1_END do
	table.insert(demo, {_M.tally_leds[i].red.drv, ON})
	table.insert(demo, {_M.tally_leds[i].green.drv, OFF})
	local x = i + GROUP_2_START - 1
	if x <= GROUP_2_END then
		table.insert(demo, {_M.tally_leds[x].red.drv, ON})
		table.insert(demo, {_M.tally_leds[x].green.drv, OFF})
	end
	--Hold 1 step.
	table.insert(demo, {HOLD, 1})
end

--Round 3, all OFF step by step.
for i=GROUP_1_START, GROUP_1_END do
	table.insert(demo, {_M.tally_leds[i].red.drv, OFF})
	table.insert(demo, {_M.tally_leds[i].green.drv, OFF})
	local x = i + GROUP_2_START - 1
	if x <= GROUP_2_END then
		table.insert(demo, {_M.tally_leds[x].red.drv, OFF})
		table.insert(demo, {_M.tally_leds[x].green.drv, OFF})
	end
	--Hold 1 step.
	table.insert(demo, {HOLD, 1})
end

_M.on_demo.stages["restore"] = demo

-- The LED animation when quick reset.
demo = {}
--Round 1, green ON red OFF.
table.insert(demo, {_M.fan[1].drv, 100}) --Fan 100% run.
for i=GROUP_1_START, GROUP_1_END do
	table.insert(demo, {_M.tally_leds[i].red.drv, OFF})
	table.insert(demo, {_M.tally_leds[i].green.drv, ON})
	local x = i + GROUP_2_START - 1
	if x <= GROUP_2_END then
		table.insert(demo, {_M.tally_leds[x].red.drv, OFF})
		table.insert(demo, {_M.tally_leds[x].green.drv, ON})
	end
	--Hold 1 step.
	table.insert(demo, {HOLD, 1})
end

--Round 2, green OFF step by step.
table.insert(demo, {_M.fan[1].drv, 0}) --Fan 100% off.
for i=GROUP_1_START, GROUP_1_END do
	table.insert(demo, {_M.tally_leds[i].red.drv, OFF})
	table.insert(demo, {_M.tally_leds[i].green.drv, OFF})
	local x = i + GROUP_2_START - 1
	if x <= GROUP_2_END then
		table.insert(demo, {_M.tally_leds[x].red.drv, OFF})
		table.insert(demo, {_M.tally_leds[x].green.drv, OFF})
	end
	--Hold 1 step.
	table.insert(demo, {HOLD, 1})
end

_M.on_demo.stages["reset"] = demo

return _M
