#!/usr/bin/env lua

local socket = require "socket"
local http = require "socket.http"
local ltn12 = require "ltn12"
local pt = require "pl.pretty"
local socket_unix = require "socket.unix"
local cjson = require "cjson"
local coaf = require "coaf.core"

coaf.ChDir(coaf.GetAppDir(arg))

local _M = {}

function api(path, host, method, request_body, timeout)
    -- print(path, host, method, request_body, timeout)
    http.TIMEOUT = timeout or 5
    if not host then
        host = ""
    end
    if not path then
        path = ""
    end
    local source = nil
    local headers = nil

    if request_body then
        headers = {
            ["Content-Type"] = "application/json",
            ["Content-Length"] = #request_body,
            ["host"] = "null",
        }
        source = ltn12.source.string(request_body)
    else
        headers = {["host"] = "null"}
    end

    local response_body = {}
    if not method then
        method = "GET"
    end

    local create = nil
    local url = nil
    
    if host == "/var/run/docker.sock" then  --注意url和path!!!
        create = socket_unix
        url = path
        path = nil
    else
        url = host .. path
        host = nil
        path = nil
    end

    local request_data = {
        url = url,
        host = host,
        path = path,
        method = method,
        scheme = "http",
        headers = headers,
        source = source,
        sink = ltn12.sink.table(response_body),
        create = create,
    }

    local res, code, response_headers = http.request(request_data)
    -- print(res, code, response_headers)

    local ret = 1
    if res and code then
      print(table.concat(response_body))
      if code == 200 then
        ret = 0
      end
    end
    return ret
end


local host = nil
local path = nil
local request_body = nil
local method = nil
local timeout = nil
if type(arg) ~= "table" or #arg < 2 then
	os.exit(1)
end

host = arg[1]
path = arg[2]
request_body = arg[3] or ""
method = arg[4] or "GET"
timeout = arg[5] or 15
--[[
print(host,path,method,request_body,timeout)
local params = {
  id = "12345678901234",
  authcode = "QURQNRWF3P"
}
local request_body = cjson.encode(params)
--]]
local ret = api(path,host,method,request_body,timeout)
os.exit(ret or 1)
