#!/usr/bin/env lua
local coaf = require 'coaf.core'
local _dbg = require 'coaf.debug'

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require 'cosocket.core'
local client = require 'coaf.client'
local adapter = require 'coaf.cli_adapter.simple'
local service = require 'coaf.service'
local pt = require 'pl.pretty'
local thread = require "coaf.thread"

local event = require 'coaf.event'

local withLog = false
local _x
for _x = 1, #arg do
    if arg[_x] == '--log' then
        withLog = true
        break
    end
end

if withLog then
    -- ** initialize the log file **
    local roll = require 'coaf.log.rollfile'
    os.execute('/bin/mkdir -p /var/log/')
    local logF = roll.Create('/var/log/mediaserver.log')
    if not logF or not logF:init() then
        _dbg.Error(
            'mediaserver',
            'Fail to open log file: /var/log/mediaserver.log, so the log messages should be output to console.'
        )
        withLog = false
    else
        _dbg.AddLogger('logfile', logF, nil, nil)
    end
-- ** log **
end

coaf.Init('mediaserver')

if withLog then
    local stdio = require 'coaf.log.stdio'
    stdio.SetDefaultFilterName('mediaserver')
    stdio.Start()
end

service.InitSocket()
client.InitSocket(socket, adapter)


_G.g_event = event.createNew('mediaserver')
local mo_streams = require 'mod_Streams'
local mo_Streamer = require 'mod_Streamer'
local mo_Webrtc = require "mod_Webrtc"
local mo_cameraLink = require "mod_cameraLink"


local function for4000ms_PeriodCheck()
	--if not for200ms_checking_flag then
		--for200ms_checking_flag = true
	local r,msg = pcall( function()
		if mo_Streamer then
			mo_Streamer.Step()
		end
	end )
	if not r then
		_dbg.Error("CODEC", "Period checking error:", msg)
	end
end



-- ** CoService **
local cosvc_Server
------------------

-- **5s period checker**
local server_refresh_counter = 0

_G.cfg_tun = require('tun_config').Load()
if not _G.cfg_tun then
    _dbg.Error('mediaserver', 'Load coturn config error!')
    _G.cfg_tun = {}
end
local for4000msTimer

-- **Main entry**
local function Main(args)
    mo_streams.Init()
    mo_Webrtc.Init()
    mo_cameraLink.Init()

    cosvc_Server = service.New('mediaserver', 'unix:/var/run/mediaserver.sock')
    if cosvc_Server then
        _dbg.Info('mediaserver', "CoService of 'mediaserver' is created.")
        cosvc_Server:CreateCoServerProxy()
        cosvc_Server:ExposeObject('/streams', mo_streams)        
    else
        _dbg.Error('mediaserver', 'Fail created CoService!')
    end

    mo_Streamer.Init(cosvc_Server)

    if cosvc_Server then
        cosvc_Server:ExposeObject('/streamer', mo_Streamer) 
        cosvc_Server:ExposeObject("/webrtc", mo_Webrtc )
        cosvc_Server:ExposeObject("/cameraLink", mo_cameraLink )

    end

    for4000msTimer = thread.NewTimer( for4000ms_PeriodCheck, 0.01, 4 )
    if for4000msTimer then
		for4000msTimer:start()
	end



end

coaf.Start(Main, arg)


if for4000msTimer then
    for4000msTimer:stop()
end

mo_streams.Destory()

_dbg.Info('mediaserver', 'mediaserver is destroying.')
event.free('mediaserver')
_dbg.Print('mediaserver', 'Service terminated.')
