local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

require "kmCore"

local _M = {
	created_streams = {}
}

---------------------------------------------

local function decoderEventCallback()
	--TODO:
end

function _M.ConstructStreamMeta( stream_meta )
----------------------------------------------------------------
--{ Construct meta here!
----------------------------------------------------------------
stream_meta = stream_meta or {}

function stream_meta:Init()
	self.stream_openned = false

	_M.created_streams[ self.id ] = self
	--Step1: Create media synchronizer
	_dbg.Info( "STREAM", self.id, ": Create media synchronizer" )
	self.syncSource = KM_STREAMING.SyncSource.createNew()
	self.syncSource:SetBufferOptions( self.buffer_option )

	if self.SpecialInit then
		if not self:SpecialInit() then
			return false
		end
	end
	return true
end


function stream_meta:Destroy()
    self:Stop()
    
	if self.syncSource then
		self.syncSource:BindMediaSource(nil)
		self.syncSource = nil
	end

	if self.SepcialDestroy then
		self:SpecialDestroy()
	end
	_M.created_streams[ self.id ] = nil

	self.start_actions = {}
	collectgarbage()
end

function stream_meta:CheckStartActions()
	local rslt = {}
	local withActions = false
	local group, actions
	for group, actions in pairs(self.start_actions) do
		local act, v
		for act, v in pairs(actions) do
			rslt[act] = true
			withActions = true
		end
	end
	return withActions, rslt
end

function stream_meta:SetBufferOptions(opt)
	if opt and opt ~= "" then
		self.buffer_option = opt
	end

	_dbg.Info( "STREAM", self.id, "Change the buffer options:", opt )

	if self.syncSource then
		self.syncSource:SetBufferOptions( self.buffer_option )
	end
end

function stream_meta:Start(group, action)
	local new_with_act, new_actions
	if group then
		if not action or action == "" or action == "all" then
			self.start_actions[group] = { stream=true, output=true }
		else
			if not self.start_actions[group] then
				self.start_actions[group] = { [action] = true }
			else
				self.start_actions[group][action] = true
			end
		end
	end
	new_with_act, new_actions = self:CheckStartActions()
	if not new_with_act then
		return true
	end

	local stream_start = false

	local act, v
	for act, v in pairs(new_actions) do
		stream_start = true
		break
	end

	if stream_start and not self.stream_openned then
		if self.SpecialStart then
			self:SpecialStart()
		end
		self.stream_openned = true
	end

	return true
end

function stream_meta:Stop(group, action)
	local new_with_act, new_actions
	if group then
		if not action or action == "" or action == "all" then
			self.start_actions[group] = nil
		else
			if self.start_actions[group] then
				self.start_actions[group][action] = nil
			end
		end
	end

	local stream_start = false

	if group then
		new_with_act, new_actions = self:CheckStartActions()
		if new_with_act then
			local act, v
			for act, v in pairs(new_actions) do
				stream_start = true
			end
		end
	end

	if not stream_start and self.stream_openned then
		if self.SpecialStop then
			self:SpecialStop()
		end
		self.stream_openned = false
	end
	return true
end

stream_meta.__index = stream_meta

return stream_meta
----------------------------------------------------
-- } End of construct meta.
----------------------------------------------------
end

function _M.Init(event, event_name)
	_M.eventObj = event
	_M.eventName = event_name
end

function _M.CheckStreamExist(id)
	return _M.created_streams[id]
end

return _M
