local mod_GenStream = require 'mod_GeneralStream'

local coaf = require 'coaf.core'
local _dbg = require 'coaf.debug'

require 'kmCore'

local _M = {}

local stream_meta = {}

--remove ndi://
local function FormatURL(url)
	local protocol, orig_url_tail

	protocol, orig_url_tail = url:match( "(%w+)://(.+)$" )
	if not protocol then
		return  url
	else
		if protocol == "ndi" or protocol == "NDI" then
			return orig_url_tail
		else
			_dbg.Error('NDI_RECV','NDI URL ERROR! URL=', url)
			return nil
		end
	end
end

function stream_meta:SpecialInit()
    self.ndiRecv = KM_STREAMING.ndiMediaSource.createNew()
    if not self.ndiRecv then
        _dbg.Error('NDI_RECV', self.id, ' fail to create NDI recv service')
        self.Destroy()
        return false
    end

    self.syncSource:BindMediaSource(self.ndiRecv:GetMediaSource(), true)
    return true
end

function stream_meta:SpecialDestroy()
    self.ndiRecv = nil
    _dbg.Info('NDI_RECV', self.id, ': destroy NDI RECV service')
    return true
end

function stream_meta:CheckURLAndOptions(url, options)
    local stream_options = self.stream_options or {}
 
    local updated = false
	--TODO:
	if stream_options.ndi_name ~= options.ndi_name then
		stream_options.ndi_name = options.ndi_name
		updated = true
	end

	if stream_options.preview_mode ~= options.preview_mode then
	    stream_options.preview_mode = options.preview_mode
		updated = true
	end

	if stream_options.discovery_server ~= options.discovery_server then
		stream_options.discovery_server = options.discovery_server
		updated = true
	end

	self.stream_options = stream_options

    return updated, url
end

function stream_meta:ChangeSource(url, options)
    local updated
    url = url or self.url or ''
    if options then
        updated, url = self:CheckURLAndOptions(url, options)
        if not url then
            return
        end
    end

    if self.url ~= url or updated then
        self.url = url
        if not self:CheckStartActions() then
            return
        end
        self:Stop(nil, true) --Stop all but don't free decoder.
        self:Start()
    end
end

function stream_meta:SpecialStart(group, action)
	local ndi_url = FormatURL(self.url)
	if ndi_url then
		self.ndiRecv:OpenStream(ndi_url, self.stream_options)
		_dbg.Info('NDI_RECV', self.id, ': Started NDI RECV service,  url is :', self.url)
	end
end

function stream_meta:SpecialStop(group, action)
    self.ndiRecv:CloseStream()
    _dbg.Info('NDI_RECV', self.id, ': Stop NDI RECV service')
    return true
end

function stream_meta:GetStreamStatus(group)
    local with_act, acts = self:CheckStartActions()
    local output_start = false
    local stream_start = false
    if with_act then
        local k, v
        for k, v in pairs(acts) do
            if k == 'output' then
                output_start = true
                stream_start = true
                break
            else
                stream_start = true
            end
        end
    end

    local ret = {
        output_start = output_start,
        stream_start = stream_start
    }

    if not self.ndiRecv then
        ret.live = 'closed'
        ret.resolution = '?x?'
        ret.bitrate = 0
        ret.video = 'NONE'
        ret.audio = 'NONE'
        ret.frameRate = 0
    else
        local status = self.ndiRecv:GetStatus()
        if not status then
            ret.live = 'error'
            ret.resolution = '?x?'
            ret.bitrate = 0
            ret.video = 'NONE'
            ret.audio = 'NONE'
            ret.frameRate = 0
        else
            ret.live = status.live and status.live.status or 'error'
            if status.video then
                ret.resolution =
                    '' ..
                    (status.video.width or '0') ..
                        'x' .. (status.video.height or '0') .. '@' .. (status.video.frameRate or '0') .. 'Hz'
                ret.bitrate = status.video.bitrate or 0
                ret.video = status.video.codec or 'H264'
                ret.frameRate = status.video.frameRate or 0
            else
                ret.video = 'NONE'
                ret.resolution = '?x?'
                ret.bitrate = 0
                ret.frameRate = 0
            end

            if status.audio then
                ret.audio = status.audio.codec or 'AAC'
                ret.bitrate = ret.bitrate + (status.audio.bitrate or 0)
            else
                ret.audio = 'NONE'
            end
        end
    end
    return ret
end

function stream_meta:GetMediaSource()
    --if self.syncSource then
    --      return self.syncSource:GetMediaSource()
    if self.ndiRecv then
        return self.ndiRecv:GetMediaSource()
    else
        return nil
    end
end

stream_meta = mod_GenStream.ConstructStreamMeta(stream_meta)

----------------------------------

function _M.Init()
end

function _M.Create(id, buffer_option)
    local stream = mod_GenStream.CheckStreamExist(id)
    if stream then
        _dbg.Warn('NDI_STREAM', "The creating stream '" .. id .. "' is already exist.")
        return stream
    end

    if not buffer_option or buffer_option == '' then
        buffer_option = 'live:200:200:500:0:0'
    end

    stream = {
        id = id,
        type = 'ndi',
        decoder_group_id = "min_group",
        decoderGroup = "decoder_group",
        decoder_channel = nil,
        start_actions = {},
        audio_outputs = {},
        audio_enabled = {},
        vpss_using = {},
        current_audio_intfs = {},
        stream_openned = false,
        buffer_option = buffer_option,
        url = nil,
		stream_options = nil,
        origin = "manual"
    }

    setmetatable(stream, stream_meta)
    stream:Init()
    return stream
end

return _M
