local mod_GenStream = require "mod_GeneralStream"
local CO = require "mod_coservice"

local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

require "kmCore"

local _M = {
}

---------------------------------------------
local function  AddRtmpSession(opts)
    local proxy = CO.GetProxy("/rtmp_service", "rtmp_service")
    if not proxy then
        _dbg.Error("RtmpServer", "Connect rtmp service error")
        return false
	else
	local pretty = require "pl.pretty"
	pretty.dump(opts)
	
        local r, ret = proxy.AddSession(opts)
        proxy:Destroy()
        if not r or not ret then
            _dbg.Error("RtmpServer", "Add rtmp session error!")
            return false
        else
            return true
        end
    end
end

local function  RemoveRtmpSession(session)
    local proxy = CO.GetProxy("/rtmp_service", "rtmp_service")
    if not proxy then
        _dbg.Error("RtmpServer", "Connect rtmp service error")
        return false
    else
        local r, ret = proxy.RemoveSession(session)
        proxy:Destroy()
        if not r or not ret then
            _dbg.Error("RtmpServer", "Remove rtmp session error!")
            return false
        else
            return true
        end
    end
end

local function  GetRtmpSessionStatus(session)
    local proxy = CO.GetProxy("/rtmp_service", "rtmp_service")
    if not proxy then
        _dbg.Error("RtmpServer", "Connect rtmp service error")
        return nil
    else
        local r, ret = proxy.GetSessionStatus(session)
        proxy:Destroy()
        if not r or not ret then
            _dbg.Error("RtmpServer", "Update rtmp server port error!")
            return nil
        else
            return ret
        end
    end
end

local stream_meta = {}

function stream_meta:SpecialInit()
	--Step2: Create network media source
	self.networkSource = KM_STREAMING.networkSource.createNew()
	if not self.networkSource then
		_dbg.Error( "RTMP_SERVER_STREAM", self.id, ": Fail to create network media puller" )
		self:Destroy()
		return false
	end

	self.syncSource:BindMediaSource( self.networkSource:GetMediaSource(), true ) --Since previous is just only network source, I can reuse the crossbuffer
	return true
end

function stream_meta:SpecialDestroy()
	if self.url then
		local session = self:GetAppSessionByUrl(self.url)
		if session and session ~= "" then
			RemoveRtmpSession(session)
		end
	end
	self.networkSource = nil
	self.local_rtmp_url = nil
end

function stream_meta:GetAppSessionByUrl(url)
	local base_url = ""

	if not url then
		return base_url
	end

	local _, s = url:match( "(%w+)://(.+)$" )
	if not s then
		return base_url
	end
		
	local _, app   = s:match( "(.-)/(.+)$" )
	if app then
		base_url = "/" .. app
	end
	return base_url
end


function stream_meta:CreateBaseUrl(application, session, token)
	local base_url = ""
	if application  and application ~= "" then
		base_url = base_url .. "/" .. application
	end

	if session and session ~= "" then
		base_url = base_url .. "/" ..session
	end

	if token and token ~= "" then
		base_url = base_url .. "/" ..token
	end
	return base_url
end


function stream_meta: ChangeRtmpServer(options)
	--local pretty = require "pl.pretty"
	--pretty.dump(options)
	
	local stream_options = self.stream_options or {}
	options = options or {}
	local old_session,session
	if stream_options.rtmp_server_enable ~= options.rtmp_server_enable or stream_options.url ~= options.url then
		old_session = self:CreateBaseUrl(stream_options.rtmp_server_application,
										 stream_options.rtmp_server_session,
										 stream_options.rtmp_server_token)
		if old_session and old_session ~= "" then
			RemoveRtmpSession( old_session) 
		end

		session = self:CreateBaseUrl(options.rtmp_server_application,
									 options.rtmp_server_session,
									 options.rtmp_server_token)
		if session and session ~= "" then
			AddRtmpSession({url = session, isEnable = (options.rtmp_server_enable and 1 or 0) })
		end
		
		stream_options.rtmp_server_enable = options.rtmp_server_enable or false
		stream_options.url = options.url or nil
	end
	self.stream_options = stream_options
end 

function stream_meta:ChangeRtmpSource(url, options)
	--local pretty = require "pl.pretty"
	--pretty.dump(stream_options)
	--pretty.dump(options)
	local updated = false
	options = options or {}
	local stream_options = self.stream_options or {}
	stream_options.read_timeout = 5000000
	stream_options.connect_timeout = 5000000
	stream_options.connIntv = 3000000
	stream_options.rtmp_server_port = options.rtmp_server_port
	stream_options.rtmp_server_token = options.rtmp_server_token
	stream_options.rtmp_server_session = options.rtmp_server_session
	stream_options.rtmp_server_security = options.rtmp_server_security
	stream_options.rtmp_server_application = options.rtmp_server_application
	stream_options.url = options.url

	-- network stream info analyse speed
	local con_speed = options.connect_speed or 2
	if stream_options.connect_speed ~= con_speed  then
		stream_options.connect_speed = con_speed
		updated = true
	end

	if con_speed == 0 then
		stream_options.analyzeduration = 1000000   --1s
		stream_options.probesize 	   = 128*1024  --128K
	elseif con_speed == 1 then
		stream_options.analyzeduration = 2000000
		stream_options.probesize 	   = 2000000
	elseif con_speed == 3 then
		stream_options.analyzeduration = 8000000
		stream_options.probesize	   = 8000000
	else
			--nothing todo
	end
	self.stream_options = stream_options

	local rtmp_url = "rtmp://127.0.0.1"
	if stream_options.rtmp_server_port then
		rtmp_url = rtmp_url .. ":" .. stream_options.rtmp_server_port
	end

	if stream_options.rtmp_server_application then
		rtmp_url = rtmp_url .. "/" .. stream_options.rtmp_server_application
	end

	if stream_options.rtmp_server_session then
		rtmp_url = rtmp_url .. "/" .. stream_options.rtmp_server_session
	end

	if stream_options.rtmp_server_security and stream_options.rtmp_server_security ~= "none" 
	and stream_options.rtmp_server_token and stream_options.rtmp_server_token ~= "" then
		rtmp_url = rtmp_url .. "/" .. stream_options.rtmp_server_token
	end


	if self.url ~= url or self.local_rtmp_url ~= rtmp_url or updated then
		self.url = url
		self.local_rtmp_url = rtmp_url
		if not self:CheckStartActions() then
			return
		end
		self:Stop(nil, true) --Stop all but don't free decoder.
		self:Start()
	end
end

function stream_meta:ChangeSource(url,options)
	if not options  or not url then
		_dbg.Error( 'RTMP_SERVER_STREAM', self.id, "Invalid rtmp opts type of:", url )
		return false
	end

	local protocol, s = url:match( "(%w+)://(.+)$" )
	if not protocol or  not s then
		print('*** ERROR: Invalid rtmp URL=',url)
		return false
	end

	--此处分为rtmp拉流参数和rtmp server 参数两部分判断
	self:ChangeRtmpServer(options)
	self:ChangeRtmpSource(url, options)
end

function stream_meta:SpecialStart(group, action)
	self.networkSource:Open( self.local_rtmp_url, self.stream_options )
	_dbg.Info( "RTMP_SERVER_STREAM", self.id, ": Created stream puller for url:", self.local_rtmp_url )
	return true
end

function stream_meta:SpecialStop(group, action)
	self.networkSource:Close()
	_dbg.Info( "RTMP_SERVER_STREAM", self.id, ": Destroied stream puller" )
	return true
end

function stream_meta:GetStreamStatus(group)
	local with_act, acts = self:CheckStartActions()
	local output_start = false
	local stream_start = false
	if with_act then
		for k,v in pairs(acts) do
			if k == "output" then
				output_start = true
				stream_start = true
				break
			else
				stream_start = true
			end
		end
	end

	local ret = {
		output_start = output_start,
		stream_start = stream_start
	}

	if not self.networkSource then
		ret.live = "closed"
		ret.resolution = "?x?"
		ret.bitrate = 0
		ret.video = "NONE"
		ret.audio = "NONE"
		ret.frameRate = 0
	else
		local status = self.networkSource:GetStatus()
		if not status then
			ret.live = "error"
			ret.resolution = "?x?"
			ret.bitrate = 0
			ret.video = "NONE"
			ret.audio = "NONE"
			ret.frameRate = 0
		else
			ret.live = status.live and status.live.status or "error"
			if status.video then
				ret.resolution = '' .. (status.video.width or "0") .. "x" .. (status.video.height or "0") .. "@" .. (status.video.frameRate or "0") .. "Hz"
				ret.bitrate = status.video.bitrate or 0
				ret.video = status.video.codec or "H264"
				ret.frameRate = status.video.frameRate or 0
			else
				ret.video = "NONE"
				ret.resolution = "?x?"
				ret.bitrate = 0
				ret.frameRate = 0
			end

			if status.audio then
				ret.audio = status.audio.codec or "AAC"
				ret.bitrate = ret.bitrate + (status.audio.bitrate or 0)
			else
				ret.audio = "NONE"
			end
		end
	end

	return ret
end

function stream_meta:GetRtmpSessionStatus()
	if  self.url then
		local session = self:GetAppSessionByUrl(self.url)
		if session and session ~= "" then
			return GetRtmpSessionStatus(session)
		end
	else
		return nil
	end
end

function stream_meta:GetMediaSource()
	if self.networkSource then
		return self.networkSource:GetMediaSource()
	else
		return nil
	end
end

stream_meta = mod_GenStream.ConstructStreamMeta( stream_meta )

----------------------------------

function _M.Init()
end

function _M.Create(id, buffer_option, islocal)
	local stream = mod_GenStream.CheckStreamExist(id)
	if stream then
		_dbg.Warn( "RTMP_SERVER_STREAM", "The creating stream '" .. id .. "' is already exist." )
		return stream
	end

	if not buffer_option or buffer_option == "" then
		buffer_option = "live:200:200:500:0:0"
	end

	if not islocal  then
		islocal = false
	end

	stream = {
		id = id,
		type = "rtmp_server",
		decoder_group_id = "min_group",
		decoderGroup = "decoder_group",
		decoder_channel = nil,
		start_actions = {},
		audio_outputs = {},
		audio_enabled = {},
		vpss_using = {},
		current_audio_intfs = {},
		stream_openned = false,
		buffer_option = buffer_option,
		url = nil,
		stream_options = nil,
		islocal = islocal,
		local_rtmp_url = nil
	}

	setmetatable( stream, stream_meta )
	stream:Init()
	return stream
end

return _M

