local coaf = require 'coaf.core'
local _dbg = require 'coaf.debug'
local net_stream = require 'mod_NetStream'
local srt_recv = require 'mod_SrtRecv'
local rtmp_srv_stream = require "mod_RtmpSrvStream"
local ndi_recv = require 'mod_NdiRecv'

local _done, features = pcall(require, 'features')
if not _done then
	features = {}
elseif type(features) ~= "table" then
	features = _G
end

local CO = require "mod_coservice"

local _M = {
	order_counter = 10,
	dummy_s = nil,
	streams = {},
	sub_streams = {},
	localstreams_config = {},
	stream_order = {},
	streams_config = nil,
	manual_id_base = 1,
	outputstream = {},
}

--[[
local function GetLocalEnInfo()
	local proxy = CO.GetProxy( "/local", "CoManager" )
	if not proxy then
		_dbg.Error("STREAMS", "CoManager connect error" )
		return false
	else
		local r,Data = proxy.GetLocalEnInfo()
		proxy:Destroy()
		if not r or not Data then
			_dbg.Error("STREAMS", "GetLocalEnInfo error" )
			return false
		else
			return Data
		end
	end
	return false
end
]]

--老的配置文件的标志 含有sub_url和 manual_url
--由于localstream 只含有RTSP简单转换一下
function  _M.UpdateLocalStreamCfg(old_cfg)
	local new_cfg = {}
	local type = old_cfg.options and old_cfg.options.strm_type  or ""
	new_cfg.name          = old_cfg.name or ""
	new_cfg.manual        = old_cfg.manual or false
	new_cfg.manual_type   = old_cfg.origin or "local"
	new_cfg.audio_enables = old_cfg.audio_enables or {}
	new_cfg.with_sub      = old_cfg.with_sub or true
	new_cfg.sub_options= {
		type = "rtsp",
		buffer_option = old_cfg.buffer_option or "live:100:120:200:0:0",
		transport = "tcp",
		connect_speed = 2,
		url = old_cfg.sub_url or "rtsp://",
		imgUrl = old_cfg.options and old_cfg.options.imgUrl  or "",
		username =  old_cfg.options and old_cfg.options.username  or "",
		password =  old_cfg.options and old_cfg.options.password  or ""
	}

	new_cfg.options = {
		type = type or "rtsp",
		buffer_option = old_cfg.buffer_option or "live:100:120:200:0:0",
		transport     = "tcp",
		connect_speed = 2,
		url      = old_cfg.manual_url  or "rtsp://",
		imgUrl   =  old_cfg.options and old_cfg.options.imgUrl  or "",
		username =  old_cfg.options and old_cfg.options.username  or "",
		password =  old_cfg.options and old_cfg.options.password  or ""
	}
	return new_cfg
end

function  _M.UpdateNetStreamCfg(old_cfg)
	local new_cfg = {}
	local _type   =  old_cfg.type and old_cfg.type or (old_cfg.protocol or "rtsp")
	new_cfg.name          = old_cfg.name or ""
	new_cfg.manual        = old_cfg.manual or true
	new_cfg.manual_type   = old_cfg.manual_type or "manual"
	new_cfg.audio_enables = old_cfg.audio_enables or {}
	new_cfg.with_sub      = old_cfg.with_sub or false
	new_cfg.sub_options = {
		type = "rtsp",
		buffer_option = "live:200:200:500:0:0",
		transport = "tcp",
		connect_speed = 2,
		url = "rtsp://",
	}

	--print("Debug:--_type",_type)
	
	--根据复杂的类型构建各种兼容的推流参数
	if _type:lower() == "srt" then
		new_cfg.options = {
			type = "srt",
			buffer_option = old_cfg.buffer_option or  "live:200:200:500:0:0",
			url = "srt://" .. old_cfg.options.address .. ":" .. old_cfg.options.listenerPort,
			connect_speed = old_cfg.options.connect_speed or 2,
			encryption    =  old_cfg.options.encryption  or "" ,
			passphrase    =  old_cfg.options.passphrase  or "",
			connectionMode =  old_cfg.options.connectionMode or "caller",
			latency     =  old_cfg.options.latency or 125,
			payloadSize =  old_cfg.options.payloadSize or 1316,
			bandwidth   =  old_cfg.options.bandwidth or 25,
			address     = old_cfg.options.address,
			listenerPort = old_cfg.options.listenerPort,
			srt_streamid = ""
		}
	elseif 	_type:lower() == "file" then
		new_cfg.options = {
			type          = "file",
			buffer_option = old_cfg.buffer_option or  "live:200:200:500:0:0",
			file_list     = old_cfg.file_list or {},
			loop          = old_cfg.options.loop or false
		}
	elseif 	_type:lower() == "ndi" then
		new_cfg.options = {
			type          = "ndi",
			url = "ndi://" .. (old_cfg.manual_url or ""),
			buffer_option = old_cfg.buffer_option or  "live:200:200:500:0:0",
			preview_mode  = old_cfg.options.preview_mode or 0,
			ndi_name      = old_cfg.options.ndi_name or false
		}
	else
		local _url = old_cfg.manual_url or ""
		if not _url then
			_dbg.Error( "STREAMS", "Fail to update streams configure file -> _url is null!" )
			return nil
		end

		local protocol, orig_url_tail
		protocol, orig_url_tail = _url:match( "(%w+)://(.+)$" ) 
		if not protocol then
			protocol = "rtsp"
		end
		--D300 文件源特殊处理
		if 	_type:lower() == "network" and protocol == "file"  then
			new_cfg.options = {
				type          = "file",
				buffer_option = old_cfg.buffer_option or  "live:200:200:500:0:0",
				file_list     = {},
				loop          = false
			}
		else
			new_cfg.options = {
				type = protocol:lower() or "rtsp",
				buffer_option = old_cfg.buffer_option or "live:200:200:500:0:0",
				transport = "tcp",
				connect_speed = 2,
				url = old_cfg.manual_url  or "rtsp://",
				username =  old_cfg.options and old_cfg.options.username  or "",
				password =  old_cfg.options and old_cfg.options.password  or "",
				rtp_audio_codec = old_cfg.options and old_cfg.options.rtp_audio_codec  or "",
				rtp_video_codec = old_cfg.options and old_cfg.options.rtp_video_codec  or "",
				rtp_audio_port = old_cfg.options and old_cfg.options.rtp_audio_port  or 0,
				rtp_video_port = old_cfg.options and old_cfg.options.rtp_video_port  or 0,
				bind           = old_cfg.options and old_cfg.options.bind  or "*"
			}
		end
	end
	return new_cfg
end

--SRT配置兼容处理
function  _M.UpdateSrtStreamCfg(old_cfg)
	local update = false
	if old_cfg.options.type:lower() == "srt" then
		if old_cfg.options.url and  not old_cfg.options.listenerPort then
			local _, ip , port  = old_cfg.options.url:match( "(%w+)://(.-):(.+)$" )
			old_cfg.options.address = ip or ""
			old_cfg.options.listenerPort = port or ""
			update = true
		end
	end

	if old_cfg.with_sub and old_cfg.sub_options.type:lower() == "srt" then
		if old_cfg.sub_options.url and  not old_cfg.sub_options.listenerPort then
			local _, ip , port  = old_cfg.sub_options.url:match( "(%w+)://(.-):(.+)$" )
			old_cfg.sub_options.address = ip or ""
			old_cfg.sub_options.listenerPort = port or ""
			update = true
		end
	end

	if update then
		return old_cfg
	else
		return nil
	end
end

function _M.Init()
	local cfg_streams = require "streams_config"
	_M.streams_config = cfg_streams.Load()
	if not _M.streams_config then
		_dbg.Error( "STREAMS", "Fail to load streams configure file!" )
		_M.streams_config = {
			__TRIGGER = function() end
		}
	end

	--启动配置文件升级
	local update_cfg = false
	for k, old_cfg in pairs(_M.streams_config) do
		local new_cfg
		if k:match("LOCAL.*$") then
			if old_cfg.sub_url and old_cfg.manual_url then
				new_cfg =  _M.UpdateLocalStreamCfg(old_cfg)
			end
		elseif  k:match("MANUAL.*$") then
			if (old_cfg.sub_url and (old_cfg.manual_url or old_cfg.file_list)) or old_cfg.protocol then
				new_cfg =  _M.UpdateNetStreamCfg(old_cfg)
			end
		end

		--此处说明是较新的配置，只需更新address和port
		if not new_cfg then
			new_cfg = _M.UpdateSrtStreamCfg(old_cfg)
		end

		if new_cfg then
			_M.streams_config[k] = new_cfg
			update_cfg = true
		end
	end
	if update_cfg then
		_M.streams_config:__TRIGGER(true)
	end

	local my_type
	local max_manual_id, id
	for k, set in pairs(_M.streams_config) do
		_dbg.Info( "STREAMS", "Init", k )

		--注意此处包含远程添加的设备初始化 manual_type="remote"
		if set.manual then
			id = k:match( "MANUAL.(%d+)" )
			if id then
				id = tonumber(id)
				if not max_manual_id or id > max_manual_id then
					max_manual_id = id
				end
			end
			my_type = set.manual_type
			_dbg.Info( "STREAMS", "Init manual", k, my_type )
		else
			my_type = "not-netstream"
		--TODO: 要处理SRT流的情况！！！
		end		

		--处理手动添加的网络流 local流在其他模块添加
		if my_type ~= "not-netstream" then
			if set.options and set.options.type then 
				if set.options.type:lower() == "srt" then
					_M.streams[k] = srt_recv.Create(k, set.options.buffer_option)
				elseif set.options.type:lower() == "ndi" then
					_M.streams[k] = ndi_recv.Create(k, set.options.buffer_option)
				elseif set.options.type:lower() == "rtmp_server" then
					_M.streams[k] = rtmp_srv_stream.Create(k, set.options.buffer_option)
				else
					_M.streams[k] = net_stream.Create( k, set.options.buffer_option )
				end
			end

			if not _M.streams[k] then
				_dbg.Error( "STREAMS", "Fail to create manual set stream of:", k, ", url=", set.options.url )
			else
				-- _dbg.Info( "STREAMS", "create manual set stream of:", k, ", url=", set.manual_url )
				_M.streams[k]:ChangeSource( set.options.file_list or set.options.url, set.options)

				if type(set.audio_enables) == "table" then
					local grp,en
					for grp,en in pairs(set.audio_enables) do
						_M.streams[k]:EnableAudioOutput(grp,en)
					end
				end

				_M.streams[k].origin = my_type
				_M.streams[k].imgUrl = set.options.imgUrl or ""
				_M.stream_order[k] = _M.order_counter
				_M.order_counter = _M.order_counter + 1
			end

			--Create sub-streams.
			if set.with_sub and set.sub_options and  set.sub_options.type then
				if set.sub_options.type:lower() == "srt" then
					_M.sub_streams[k] = srt_recv.Create(k .. ":sub", set.sub_options.buffer_option)
				elseif set.sub_options.type:lower() == "ndi" then
					_M.sub_streams[k] = ndi_recv.Create(k .. ":sub", set.sub_options.buffer_option)
				elseif set.sub_options.type:lower() == "rtmp_server" then
					_M.sub_streams[k] = rtmp_srv_stream.Create(k .. ":sub", set.sub_options.buffer_option)
				else
					_M.sub_streams[k] = net_stream.Create( k .. ":sub", set.sub_options.buffer_option )
				end

				if not _M.sub_streams[k] then
					_dbg.Error( "STREAMS", "Fail to create manual set sub-stream of:", k, ", url=", set.sub_options.url )
				else
					-- _dbg.Info( "STREAMS", "create manual set sub-stream of:", k, ", url=", set.sub_url )
					_M.sub_streams[k]:ChangeSource( set.sub_options.file_list or set.sub_options.url, set.sub_options)

					--[[ TODO:
					if type(set.audio_enables) == "table" then
						local grp,en
						for grp,en in pairs(set.audio_enables) do
							_M.sub_streams[k]:EnableAudioOutput(grp,en)
						end
					end
					--]]

					_M.sub_streams[k].origin = my_type
					_M.sub_streams[k].imgUrl = set.sub_options.imgUrl or ""

				end
			end
		end
	end

	_M.manual_id_base = (max_manual_id or 0) + 1

end

-- streamtype: {"local","manual","output","remote"}
function _M.GetStreams(streamtypes, with_sub) 
	if not streamtypes then 
		streamtypes = {"local","manual","remote"}
	end

	if streamtypes == "ALL" then
		streamtypes = {"local","manual","remote"}
	end

	if streamtypes == "LOCAL" then
		streamtypes = {"local","output"}
	end

	if type(streamtypes) ~= "table" then
		streamtypes = { streamtypes }
	end
	
	local ret = {}
	local item

	for i=1, #streamtypes do		
		for k, obj in pairs(_M.streams) do
			-- _dbg.Info( "STREAMS", "GetStreams", k, "type=", obj.type, "origin=",obj.origin )
			if obj.origin == streamtypes[i] then
				item = _M.GetStreamDetail(k,"main")
				if item then table.insert(ret, item) end

				if with_sub and _M.sub_streams[k] then
					item = _M.GetStreamDetail(k,"sub")
					if item then table.insert(ret, item) end
				end
			end
		end
	end

	table.sort( ret, function(a,b)
		--NOTE by左振宇: 注意 sort 可能出现的错误！
		return tonumber(string.match(a.id,"MANUAL.(.*)")) < tonumber(string.match(b.id,"MANUAL.(.*)"))
	end)
	return ret
end


function _M.GetStreamDetail( id,streamType )
	if id and streamType then
		local isSub
		local baseId = id
		if streamType == "sub" then
			isSub = true
		end

		local set

		local streamObj
		if isSub then
			streamObj = _M.sub_streams[id]
		else
			streamObj = _M.streams[id]
		end

		if streamObj and streamObj.origin == "local" then
			-- set = _M.localstreams_config[ id ]
			--local mo_Devices = require "mod_Devices"
			--set = mo_Devices.devices[ id ]
			--if set and _M.streams_config[id] then
				--Get configured name.
				--set.name = _M.streams_config[id].name
			--end
			-- print("GetStreamDetail ",id, streamObj.type)
			
			set = _M.streams_config[ id ]

		elseif streamObj and streamObj.origin == "output" then
			local mo_Encoder = require "mod_Encoder"
			set = mo_Encoder.local_out_encoders[ baseId ]
		else
			set = _M.streams_config[ id ]
		end

		if ( not set ) then
			-- _dbg.Error( "STREAMS", "GetStreamDetail error" )
			return nil
		end

		if not streamObj then
			return {
				id = baseId,
				name = set and set.name or ("SRC-" .. id),
				order = _M.stream_order[id] or 99999,
				type = "unknown",
				address = "UNKNOWN",
				sourceType = isSub and "sub" or "main",
				audio_enables = set and set.audio_enables or {},
				--options = set and set.options or {}
			}
		end

		local item = {
			id = baseId,
			type = streamObj.type,
			sourceType = isSub and "sub" or "main",
			name = set and set.name or streamObj.name or ("SRC-" .. id),
			manual_url =  (set and set.options) and  set.options.url  or streamObj.url,
			--buffer_option = "live:200:200:500:0:0",
			order = _M.stream_order[id] or 99900
		}

		if isSub and set.with_sub and set.sub_options then 
			item.buffer_option = set and set.sub_options.buffer_option or "live:200:200:500:0:0"
			item.manual_url = set.sub_options.url or "UNKNOWN"
		end

		if streamObj.origin == "local" then
			item.address = id
			item.islocal = true

		elseif streamObj.origin == "output" then
			item.address = "OUTPUT"
			item.islocal = true

		elseif streamObj.type == "sip" then
			item.type = streamObj:GetSipType()
			item.address = streamObj:GetShowAddress()
			item.icons = streamObj:GetIcons()

		elseif streamObj.type == "srt" then
			item.srt_status = streamObj:GetSrtStatus()
		else
			--其他网络协议
			local url
			if set.manual then
				url = isSub and set.sub_options.url or set.options.url
			else
				url = streamObj.url
			end

			item.address = url or "UNKNOWN"
			item.manual_url = url or "UNKNOWN"
		end

		-- item.origin = set.options and set.options.origin or item.origin
		item.origin = streamObj.origin
		item.imgUrl = streamObj.imgUrl or ""

		item.audio_enables = set and set.audio_enables or {}
		item.stream_info = streamObj:GetStreamStatus()

		if streamObj.GetPlayerStatus then
			item.stream_player_status = streamObj:GetPlayerStatus()
		end

		--get the status of the rtmp push session
		if streamObj.GetRtmpSessionStatus then
			item.rtmp_server_status = streamObj:GetRtmpSessionStatus()
		end

		item.file_list = streamObj.file_list

		return item
	else
		return nil
	end
end

function _M.GetStreamObject( id ,streamType)
	--TODO: test only.
	if not id or id == ""then
		return nil
	elseif not streamType or streamType == "" then
		if _M.sub_streams[id] then
			return _M.sub_streams[id]
		elseif _M.streams[id] then
			return _M.streams[id]
		else
			return nil
		end
	elseif _M.streams[id] and streamType == "main" then
		-- pt.dump(_M.streams[ id ])
		return _M.streams[ id ]
	elseif _M.sub_streams[id] and streamType == "sub" then
		return _M.sub_streams[id]
	end
	return nil
end

function _M.SetStreamAudioOutputs( id, group, intfs )
	local isSub
	if id:match(":sub$") then
		id = id:match("(.*):sub$")
		isSub = true
	end

	--TODO: 可能需要修改控制逻辑。但目前解决暂不会解子码流。
	local streamObj 
	if isSub then
		streamObj = _M.sub_streams[id]
	else
		streamObj = _M.streams[id]
	end
	if streamObj and streamObj.SetAudioOutputs then
		streamObj:SetAudioOutputs( group, intfs )
	end
end

function _M.CheckStreamTypeStatus( old_type, type)
	if not old_type or not type then
		_dbg.Warn( "STREAMS", "Stream type is null:")
		return false
	end
	if old_type ~= type and (old_type:lower() == "srt" or old_type:lower() == "ndi" or old_type:lower() == "file" or old_type:lower() == "rtmp_server" or 
	type:lower() == "srt" or type:lower() == "ndi" or type:lower() == "file" or type:lower() == "rtmp_server") then
		return true
	else
		return false
	end
end


function _M.ChangeStreamDetail( id, detail ,enable)
	--local pretty = require "pl.pretty"
    --pretty.dump(detail)
	if not id or type(detail) ~= "table" then
		return false
	end

	local isSub
	if id:match(":sub$") then
		isSub = true
		id = id:match("(.*):sub$")
	end

	local set = _M.streams_config[id]
	if not set then
		_M.streams_config[id] = detail
		set = _M.streams_config[id]
	else
		local k,v
		for k,v in pairs(detail) do
			if k == "audio_enables" then
				if type(v) == "table" then
					if not set[k] then
						set[k] = {}
					end
					local grp,en
					for grp,en in pairs(v) do
						set[k][grp] = en
					end
				end
			elseif k == "options" then
				if type(v) == "table" then
					--先把旧的主码流参数全部清理掉 
					--if not set[k] then
					set[k] = {}
					--end
					local kk,vv
					for kk,vv in pairs(v) do
						set[k][kk] = vv 
					end
				end
			elseif k == "sub_options" then
				if type(v) == "table" then
					--把旧的子码流参数全部清理掉
					--if not set[k] then
					set[k] = {}
					--end
					for kk,vv in pairs(v) do
						set[k][kk] = vv 
					end
				end
			else
				set[k] = v
			end
		end
	end
	_M.streams_config:__TRIGGER(true)

	--TODO: local streamObj = isSub and _M.sub_streams[id] or _M.streams[id]
	local streamObj = _M.streams[id]
	local subStreamObj = _M.sub_streams[id]
	local change_sub_source = false
	local remove = false
	--pretty.dump(streamObj.start_actions)

	--新增了子码流信息
	if not subStreamObj and detail.with_sub and detail.sub_options then
		subStreamObj = _M.CreateObj(id .. ":sub", false, detail.sub_options)
		if not subStreamObj then
			_dbg.Warn( "STREAMS", "Fail to create network sub-stream of ID:", id, ", url:", detail.sub_options.url )
			return false
		else
			_dbg.Info( "STREAMS", "Create a new network sub-stream of ID:", id, ", url:", detail.sub_options.url )
			subStreamObj.origin = "manual"
			subStreamObj.imgUrl = detail.sub_options.imgUrl or ""
		end
		_M.sub_streams[id] = subStreamObj
		change_sub_source = true
	end

	if subStreamObj and detail.sub_options then
		if  detail.with_sub == false then
			_dbg.Info( "STREAMS", "Destroy network sub-stream of ID:", id )
			_M.sub_streams[id] = nil
			change_sub_source = true
			remove = true
			subStreamObj:Destroy()
			subStreamObj = nil
		else
			local sub_streamtype = subStreamObj and subStreamObj.origin or "manual"
			if _M.CheckStreamTypeStatus(subStreamObj.type, detail.sub_options.type) then
				_dbg.Info( "STREAMS", "The sub stream type changes, Reset stream:", id )
				_M.sub_streams[id] = nil
				change_sub_source = true
				subStreamObj:Destroy()
				subStreamObj = nil
				subStreamObj = _M.CreateObj(id .. ":sub", false, detail.sub_options)
				subStreamObj.origin = sub_streamtype
				subStreamObj.imgUrl = detail.sub_options.imgUrl or ""
			end
			_M.sub_streams[id] = subStreamObj
		end
	end

	if streamObj and detail.options then
		if _M.CheckStreamTypeStatus(streamObj.type, detail.options.type) then
			_dbg.Info( "STREAMS", "The main stream type changes, Reset stream:", id )
			local save_actions =  streamObj.start_actions
			_M.streams[id] = nil
			streamObj:Destroy()
			streamObj = nil
			streamObj = _M.CreateObj(id, false, detail.options)
			streamObj.origin = "manual"
			streamObj.imgUrl = detail.options.imgUrl or ""
			streamObj.start_actions = save_actions
			_M.streams[id] = streamObj
		end
	end

	--其他的就是更新主码流参数
	if streamObj then
		local streamtype = streamObj.origin or "manual"
		local sub_streamtype = subStreamObj and subStreamObj.origin or "manual"

		if type(detail.audio_enables) == "table" then
			local grp,en
			local trigger_again = false

			for grp,en in pairs(detail.audio_enables) do
				--TODO: Fix this problem for sub-streams.
				if en then --If not support mixer, restrict others not output!
					local each_id, each_obj
					for each_id, each_obj in pairs(_M.streams) do
						if each_id ~= id then
							local that_set = _M.streams_config[each_id]
							if that_set and that_set.audio_enables then
								that_set.audio_enables[grp] = false
								trigger_again = true
							end
							if each_obj then
								each_obj:EnableAudioOutput(grp,false)
							end
						end
					end
				end
				streamObj:EnableAudioOutput(grp,en)
			end

			if trigger_again then
				_M.streams_config:__TRIGGER(true)
			end
		end

		if  detail.options and detail.options.buffer_option and streamObj.origin ~= "local" then
			streamObj:SetBufferOptions( detail.options.buffer_option )
			if subStreamObj and detail.sub_options and detail.sub_options.buffer_option then
				subStreamObj:SetBufferOptions( detail.sub_options.buffer_option )
			end
		end

		--if not streamObj.no_manual_url and detail.manual_url then
		if detail.options then
			if type(streamObj.ChangeSource) == "function" then
				streamObj:ChangeSource( detail.options.file_list or detail.options.url, detail.options or streamObj.stream_options  )
			end
		end

		if detail.sub_options then
			if subStreamObj and type(subStreamObj.ChangeSource) == "function" then
				subStreamObj:ChangeSource( detail.sub_options.file_list or detail.sub_options.url, detail.sub_options or subStreamObj.stream_options )
			end
		end

		-- 还原类型
		streamObj.origin = streamtype
		if subStreamObj then
			subStreamObj.origin = sub_streamtype
		end
		if true then
			--找到有act动作正在输出的流，开启对应的子码流act
			if streamObj and subStreamObj then
				for group, actions in pairs(streamObj.start_actions) do
					local act, v
					for act, v in pairs(actions) do
						if act == "output" or act == "preview" then
							subStreamObj:Start(group,"preview")
						end
					end
				end
			end
			local mo_webrtc = require "mod_Webrtc"
			mo_webrtc.UpdateStreams(id .. ":sub", remove)
		end




		local mo_streamer = require "mod_Streamer"
		mo_streamer.UpdateStreams()

		return true
	else
		return false
	end
end

function _M.CreateObj( id, islocal, options)
	local obj 
	if islocal then
		if amba_stream then
			obj = amba_stream.Create(id,  options.buffer_option or "live:100:120:200:0:0")
		end
	else
		if options.type:lower() == "srt" then
			obj = srt_recv.Create(id, options.buffer_option or "live:200:200:500:0:0")
		elseif options.type:lower() == "ndi" then
			obj = ndi_recv.Create(id, options.buffer_option or "live:200:200:500:0:0")
		elseif options.type:lower() == "rtmp_server" then
			obj = rtmp_srv_stream.Create(id, options.buffer_option or "live:200:200:500:0:0")
		else
			obj = net_stream.Create(id, options.buffer_option or "live:200:200:500:0:0")
		end
	end
	return obj
end

function _M.AddNetworkStream( id, name, options, islocal, sub_enable, sub_options )
	if not id or not options then
		_dbg.Error( "STREAMS", "Creating network stream but without valid ID!" )
		return false
	end
	
	local origin = "remote"
	if id == "MANUAL." then
		id = id .. _M.manual_id_base
		_M.manual_id_base = _M.manual_id_base + 1
		origin = "manual"
	elseif id:match("MANUAL%.") then
		origin = "manual"
	end

	if islocal then 
		origin = "local"
	end

	local streamObj = _M.streams[ id ]
	local subStreamObj = _M.sub_streams[ id ]

	if subStreamObj and sub_enable then
		subStreamObj:ChangeSource(sub_options.file_list or sub_options.url, sub_options)
		subStreamObj.origin = origin
	end

	if streamObj then
		_dbg.Info( "STREAMS", "The network stream of ID:", id, "is already exist! Just update its URL only." )
		streamObj:ChangeSource(options.file_list or options.url, options)
		streamObj.origin = origin
		return true
	end

	--注意：目前 islocal 的情况用 amba_stream 来创建流对象。可以切换音频选择。
	streamObj = _M.CreateObj(id, islocal, options)
	if not streamObj then
		_dbg.Error( "STREAMS", "Fail to create network stream of ID:", id, ", url:", options.url )
		return false
	end
	_dbg.Info( "STREAMS", "Create network stream of ID:", id, ", url:", options.url )

	streamObj:ChangeSource(options.file_list or options.url,options)
	streamObj.origin = origin

	if sub_enable and not subStreamObj then
		subStreamObj = _M.CreateObj(id .. ":sub", islocal, sub_options)
		if not subStreamObj then
			_dbg.Warn( "STREAMS", "Fail to create network sub-stream of ID:", id, ", url:", sub_options.url )
		else
			subStreamObj:ChangeSource(sub_options.file_list or sub_options.url, sub_options)
			subStreamObj.origin = origin
		end
	end

	--目前只有翼多本地源显示图片地址
	if options and options.imgUrl then
		streamObj.imgUrl=options.imgUrl
		if subStreamObj then
			subStreamObj.imgUrl = options.imgUrl
		end
	else
		streamObj.imgUrl=""
		if subStreamObj then
			subStreamObj.imgUrl = ""
		end
	end

	_M.streams[id] = streamObj
	_M.sub_streams[id] = subStreamObj

	if not _M.stream_order[id] then
		_M.stream_order[id] = _M.order_counter
		_M.order_counter = _M.order_counter + 1
	end

	-- pt.dump(streamObj)
	local set = _M.streams_config[id]
	if set then
		if name and not islocal then
			set.name = name
		end

		--if islocal then
			--Always disable audio for local streams.
		--	set.audio_enables = {}
		--else
			if type(set.audio_enables) == "table" then
				local grp,en
				for grp,en in pairs(set.audio_enables) do
					streamObj:EnableAudioOutput(grp,en)
					--TODO: subStreamObj?
				end
			end
		--end

		set.manual = not islocal
		set.manual_type = origin
		--set.manual_url = url
		set.options = options
		set.sub_options = sub_options
		set.origin = origin
		set.with_sub = (nil ~= subStreamObj)

	elseif not islocal then
		_M.streams_config[id] = {
			manual = true,
			manual_type = origin,
			name = name or ("SRC-" .. _M.stream_order[id]),
			audio_enables = {},
			options = options,
			sub_options = sub_options,
			--origin = origin,
			with_sub = (nil ~= subStreamObj),
			stream_type = "main"
		}
	else
		_M.streams_config[id] = {
			manual = false,
			manual_type = origin,
			name = name or ("SRC-" .. id),
			audio_enables = {},
			options = options,
			sub_options = sub_options,
			--origin = origin,
			with_sub = (nil ~= subStreamObj),
			stream_type = "main"
		}
	end

	_M.streams_config:__TRIGGER(true)


	local mo_streamer = require "mod_Streamer"
	mo_streamer.UpdateStreams()

	local mo_webrtc = require "mod_Webrtc"
	mo_webrtc.UpdateStreams(id, false)

	return true
end

function _M.GetStreamStatus(type)

	local mo_streamer = require "mod_Streamer"
	if not mo_streamer.pub_points then
		return {}
	end
	local point = mo_streamer.pub_points["1"]
	local streamId,streamType
	-- local pl = require "pl.pretty"
	-- pl.dump(point)
	local ndihx_data = {}
	if point and point["config"] and point["config"]["streams"] and point["config"]["streams"][1] then
		ndihx_data["enable"] = point["config"]["streams"][1]["enable"] and 1 or 0
		ndihx_data["name"] = point["config"]["streams"][1]["name"]
		ndihx_data["channel_name"] = point["config"]["streams"][1]["channel_name"]
		ndihx_data["device_name"] = point["config"]["streams"][1]["device_name"]
		ndihx_data["stream_id"] = 1
		ndihx_data["type"] = point["config"]["streams"][1]["type"]
		ndihx_data["id"] = 1
	end
	if not ndihx_data["device_name"] then
		local mo_streamer = require "mod_Streamer"
		local _,device_name = mo_streamer.GetNdiGroupName()
		if device_name then
			ndihx_data["device_name"] = device_name
		end
	end
	streamId = point.config.source_stream 
	streamType = "main"
	if type then
		streamType = type
	end
	if streamId and streamType then
		local stream
		if streamType == "main" then
			stream = _M.streams[streamId]
		elseif streamType == "sub" then
			stream = _M.sub_streams[streamId]
		end

		if stream then
			local data = stream:GetStreamStatus()
			local streams_config = _M.streams_config[streamId]
			data["stream_id"] = streamId
			data["id"] = streamId
			data["frame_rate"] = data["frameRate"]
			if streams_config then
				data["name"] = streams_config["name"] or "none"
				if streams_config["options"] then
					data["address"] = streams_config["options"]["url"] or "none"
				end
			end
			data["NDIHX"] = ndihx_data
			return data
		end
		return {NDIHX = ndihx_data}
	end
    return {NDIHX = ndihx_data}
end

function _M.test(streamid)
	local stream = _M.streams[streamid]
	local pl = require "pl.pretty"
	pl.dump(stream)
    if stream then
        return stream:Start()
    end
    return nil
end

-- new add
function _M.RemoveAllNetworkStream()
	local mo_streamer = require "mod_Streamer"
	local k,v

	for k,v in pairs(_M.streams) do
		if not _M.streams[k].islocal then
			--remove streams in streamer
			--[[
			local id,point			
			for id,point in pairs(mo_streamer.pub_points) do
				if mo_streamer.config[id] and mo_streamer.config[id].source_stream and (mo_streamer.config[id].source_stream == k or mo_streamer.config[id].source_stream == (k .. ":sub") ) then
					point:SelectSource("")
					--stop all streamer
					local sid, scfg
					for sid, scfg in pairs(point.streams) do
						point:DisableStream(sid)						
					end
				end				
			end
			]]

			_M.RemoveNetworkStream( k )
		end
	end
	return true
end

function _M.RemoveNetworkStream( id, force )
	if not id then
		_dbg.Error( "STREAMS", "Removing network stream but without valid ID!" )
		return false
	end

	if not force then
		force = false
	end
	
	local streamObj = _M.streams[ id ]
	local subStreamObj = _M.sub_streams[ id ]

	if streamObj then
		if streamObj.islocal and not force then
			_dbg.Error( "STREAMS", "Local streams did not remove!" )
			return false
		end

		_dbg.Info( "STREAMS", "Remove network stream of ID:", id )
		_M.streams[id] = nil
		_M.sub_streams[id] = nil



		local mo_streamer = require 'mod_Streamer'
		if not mo_streamer.pub_points then
			return false
		end
		local point = mo_streamer.pub_points["1"]
		if point.config.source_stream == id then
			point:SelectSource("")
		end
		-- mo_streamer.UpdateStreams()


		local mo_webrtc = require "mod_Webrtc"
		mo_webrtc.UpdateStreams(id, true)

		if streamObj.origin ~= "local" and streamObj.origin ~= "output" then --TODO:也许需要 and streamObj.type ~= "output"
			_M.streams_config[id] = nil
			_M.streams_config:__TRIGGER(true)
		end

		streamObj:Destroy()
		if subStreamObj then
			subStreamObj:Destroy()
		end
		collectgarbage()
	end
	return true
end


function _M.RemoveSipStream(id)
	if not id then
		_dbg.Error('STREAMS', 'Removing SIP stream but without valid ID!')
		return false
	end

	if not features.FEATURE_SIP and not features.FEATURE_GB28181 then
		_dbg.Error('STREAMS', 'SIP features is not supported')
		return false
	end

	local streamObj = _M.streams[id]

	if streamObj then
		_dbg.Info('STREAMS', 'Remove SIP stream of ID:', id)
		_M.streams[id] = nil


		local mo_streamer = require 'mod_Streamer'
		mo_streamer.UpdateStreams()


		if streamObj.type ~= 'local' then
			_M.streams_config[id] = nil
			_M.streams_config:__TRIGGER(true)
		end

		streamObj:Destroy()
		collectgarbage()
	end
	return true
end

function _M.AddOutputStream( id, channel, name )
	if not id then
		_dbg.Error( "STREAMS", "Creating OUTPUT stream but without valid ID!" )
		return false
	end
	
	local baseId = id
	local isSub = id:match(":sub$")
	if isSub then
		id = id:match("(.*):sub$")
	end

	local streamObj 
	if isSub then
		streamObj = _M.sub_streams[id]
	else
		streamObj = _M.streams[id]
	end

	if streamObj then
		_dbg.Info( "STREAMS", "The OUTPUT stream of ID:", baseId, "is already exist! Just update its name only." )
		--TODO: Update name?
		return true
	end

	name = name or baseId
	local mo_OutputStream = require "mod_OutputStream"
	streamObj = mo_OutputStream.Create( baseId, channel, name )
	if not streamObj then
		_dbg.Error( "STREAMS", "Invalid OUTPUT stream parameters for:", baseId, ", cound not create stream object of it!" )
		return false
	end

	if isSub then
		_M.sub_streams[id] = streamObj
	else
		_M.streams[id] = streamObj
	end

	if not _M.stream_order[id] then
		_M.stream_order[id] = _M.order_counter
		_M.order_counter = _M.order_counter + 1
	end

	--TODO: 需要保存选项吗？但是看起来不需要。
	_M.streams_config[id] = nil
	_M.streams_config:__TRIGGER(true)


	local mo_streamer = require "mod_Streamer"
	mo_streamer.UpdateStreams()


	return true
end

function _M.RemoveOutputStream( id )
	if not id then
		_dbg.Error( "STREAMS", "Removing OUTPUT stream but without valid ID!" )
		return false
	end
	
	local baseId = id
	local isSub = id:match(":sub$")
	if isSub then
		id = id:match("(.*):sub$")
	end

	local streamObj 
	if isSub then
		streamObj = _M.sub_streams[id]
	else
		streamObj = _M.streams[id]
	end

	if streamObj then
		_dbg.Info( "STREAMS", "Remove OUTPUT stream of ID:", baseId )
		if isSub then
			_M.sub_streams[id] = nil
		else
			_M.streams[id] = nil
		end

		local mo_streamer = require "mod_Streamer"
		mo_streamer.UpdateStreams()


		_M.streams_config[id] = nil
		_M.streams_config:__TRIGGER(true)

		streamObj:Destroy()
		collectgarbage()
	end
	return true
end


function _M.PlayerControl(_, id, action, seek_pos)
	local streamObj = _M.streams[id]
	if streamObj and streamObj.type == 'file' then
		local status = streamObj:GetPlayerStatus()
		if action == "seek" and (status.play or status.live_status == "media_paused") then
			if tonumber(seek_pos) and tonumber(seek_pos) <= tonumber(status.file_duration) then
				streamObj:SetSeek(tonumber(seek_pos))
			else
				return false, "Invalid seek_pos"
			end
		elseif action == "play" and not status.play then
			if status.live_status == "media_paused" then
				streamObj:Pause(false)
			else
				streamObj:Stop(nil, true)
				--NOTE: 这种情况就是播完了所有文件并且没有循环播放
				streamObj.file_idx = 1
				streamObj:Start()
			end
		elseif action == "pause" then
			if status.live_status == "media_play" then
				streamObj:Pause(true)
			end
		elseif action == "previous" then
			streamObj:Stop(nil, true)
			streamObj.file_idx = (streamObj.file_idx - 1) <= 0 and 1 or (streamObj.file_idx - 1)
			streamObj:Start()
		elseif action == "next" then
			streamObj:Stop(nil, true)
			streamObj.file_idx = (streamObj.file_idx + 1) > #streamObj.file_list and 1 or (streamObj.file_idx + 1)
			streamObj:Start()
		elseif action == "stop" then
			streamObj:Stop(nil, true)
		end
		return true
	end
	
end

function _M.DeleteFileStreamFileList(path, files)
	local update
	for id, obj in pairs(_M.streams) do
		local changed
		if obj.type == 'file' and obj.file_list then
			if type(files) == "string" and files == "*" then
					obj.file_list = {}
					update = true
					changed = true
			else
				for _, fname in ipairs(files) do
					local fix_use_idx
					local fpath = path .. fname
					local use_file = obj.file_list[obj.file_idx].url
					for idx = #obj.file_list, 1, -1 do
						local f = obj.file_list[idx]
						if f.url == fpath then
							table.remove(obj.file_list, idx)
							update = true
							changed = true
							fix_use_idx = fix_use_idx or (idx <= tonumber(obj.file_idx))
						end
					end
	
					if fix_use_idx then
						local new_idx
						for i, v in ipairs(obj.file_list) do
							if v.url == use_file then
								new_idx = i
								break
							end
						end
	
						obj.file_idx = new_idx or obj.file_idx
					end
				end
			end
		end
		if changed then
			_M.streams_config[id].file_list = obj.file_list
		end
	end
	if update then
		_M.streams_config:__TRIGGER(true)
	end
end

function _M.StopPlayBeforeInitDisk(disk)
	for id, obj in pairs(_M.streams) do
		if obj.type == 'file' and obj.file_list then
			for _, v in ipairs(obj.file_list) do
				if v.url:match(disk) then
					obj:Stop(nil, true)
				end
			end
			obj.file_list = {}
		end
	end
	_M.streams_config:__TRIGGER(true)
end

function _M.UpdateFileListTs(path, file, info)
	local check_path = path .. "/" .. file
	for id, obj in pairs(_M.streams) do
		if obj.type == 'file' and obj.file_list then
			for _, v in ipairs(obj.file_list) do
				if v.url == check_path then
					v.startTime = info.startTime
					v.stopTime = info.stopTime
					v.totalDuration = info.totalDuration
					v.totalSize = info.totalSize
				end
			end
		end
	end
	_M.streams_config:__TRIGGER(true)
end

function _M.GetNetSourceConfig(id)
	
	--if id and id:match("LOCAL.*$") then
	--	_dbg.Info( "STREAMS", " Not support get local source config！" )
	--	return nil
	--end

	local set = _M.streams_config[ id ]
	if ( not set ) then
		_dbg.Error( "STREAMS", "GetNetSourceConfig error" )
		return nil
	end
	return set
end

local function UpdateUrlPort(url, _port)
	local protocol, host, url_tail = url:match( "(%w+)://(.-)/(.+)$" )

	if not protocol or not host or not url_tail then
		return url
	end

	--maybe no port  [rtmp://192.168.2.1/app/live]
	local ip , port = host:match( "(.+):(.-)$" )
	if not ip or not port then
		ip = host
	end

	url = protocol .. "://" .. ip .. ":" .. _port .. "/" .. url_tail

	_dbg.Info( "STREAMS", "Update rtmp stream url :",url )
	return url
end

function _M.UpdateRtmpSourcePort(_port)
	if not _port then
		return false
	end

	_dbg.Info( "STREAMS", "Update rtmp service port :", _port )

	coaf.ScheduleDelayedTask(function ()
		for k, set in pairs(_M.streams_config) do
			local id =  k:match("MANUAL.*$")
			local update = false
			if id then
				if set.options and set.options.type == "rtmp_server" then
					if set.options.rtmp_server_port ~= _port then
						set.options.rtmp_server_port = _port
						set.options.url = UpdateUrlPort(set.options.url, _port)
						update = true
					end
				end

				if set.isSub and set.sub_options and set.sub_options.type == "rtmp_server" then
					if set.sub_options.rtmp_server_port ~= _port then
						set.sub_options.rtmp_server_port = _port
						set.sub_options.url = UpdateUrlPort(set.sub_options.url, _port)
						update = true
					end
				end

				if update then
					_M.ChangeStreamDetail(id,set)
				end
			end
		end
	end)
	return true
end

function _M.GetStreamName( id )
    -- pt.dump(_M.streams_config[id])
    if _M.streams_config[ id ] and _M.streams_config[ id ].name then
        return _M.streams_config[ id ].name
    end
    return nil
end

return _M
