local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local streams = require "mod_Streams"
local socket = require "socket"
--local mo_transcoder = require "mod_Transcoder"
local thread = require "coaf.thread"

require "kmCore"

local _M = {
	webRtcPusher = nil,
	all_rtc_sessions = {},
	order_counter = 0,
	thread_id = nil,
	runing = true
}


local function thread_check()
	--CheckAlive(...) or Stop()
	while _M.runing do
		for _, sessions in pairs(_M.all_rtc_sessions) do
			for index, sid in pairs(sessions) do
				if _M.webRtcPusher:CheckSession(sid.session) <= 0 then
					sessions[index] = nil
				end
			end
		end
		coaf.Sleep(1)
	end
end

------------------------------初始化---------------------------------------
function _M.Init() 
	_dbg.Info( "WEBRTC", "--Init--")
	_M.webRtcPusher = KM_STREAMING.Webrtc.Pusher.createNew("webrtc01")

	_M.thread_id = thread.New( thread_check )
	if _M.thread_id then
		_M.thread_id:Start()
	end
end

function _M.Cleanup()
	runing = false
	if _M.thread_idk then
		_M.thread_id:Terminate()
		_M.thread_id = nil
	end
end

function _M.GetArrayKey()
	_M.order_counter = _M.order_counter + 1
	return "sid-" .. tostring(_M.order_counter)
end

function _M.InsertSession(stream_id, sess_id)
	local key = _M.GetArrayKey()
	if not  _M.all_rtc_sessions[stream_id] then
		_M.all_rtc_sessions[stream_id] = {}
	end
	_M.all_rtc_sessions[stream_id][key] = {session = sess_id}
end

--[[接口说明：
		创建服务端
	参数说明：
		id：媒体源id，可以不用填写，不填写相当于服务器端只收数据，不发数据
		ip：必须填写
		mode：webrtc会话传输数据时默认采用的模式，full:全帧率、key:关键帧；(非必传，默认key)
		port：传入0表示由系统分配，传入非0表示指定端口(一定要未使用的端口
		video_type：收发视频状态，recvonly：服务器端只收视频，但不发视频；sendonly:服务器端只发视频，但不收视频；sendrecv：服务器端即发又收视频
		audio_type：收发音频状态，recvonly：服务器端只收音频，但不发音频；sendonly:服务器端只发音频，但不收音频；sendrecv：服务器端即发又收音频
--]]
function _M.CreateWebRTCServer(id, ip, mode, port, video_type, audio_type)
	port = port or 0
	video_type = video_type or "sendonly"
	audio_type = audio_type or "sendonly"
	local stream_id = id .. ":sub"
	--_dbg.Info( "WEBRTC", "CreateWebRTCServer id:", id, "port:", port )

	--优先拉取子码流进行预览，如果子码流不存在 拉取主码流	
	local streamObj = streams.GetStreamObject( stream_id )
	if not streamObj then
		_dbg.Info( "WEBRTC", "No preview stream, so use main stream id:", id)
		stream_id = id 
		streamObj = streams.GetStreamObject( stream_id )
	end
	local rtc = {}
	if streamObj then
		--if not streamObj.stream_openned then 
		--	streamObj:Start("webrtc", "stream")
		--end
		local sess_id, sdp0 =  _M.webRtcPusher:AddSessionServer( streamObj.syncSource:GetMediaSource(), {ip = ip, port = port, videoTransport = video_type, audioTransport = audio_type})
		rtc.sdp = sdp0
		rtc.id  = sess_id
		if mode then
			_M.webRtcPusher:SetWebrtcVideoSendMode( {id = sess_id, mode = mode} )
		end
		_M.InsertSession(stream_id, sess_id)
		--_dbg.Info( "WEBRTC", "server CreateWebRTCServer AddSession sess_id:", sess_id )
	else 
		_dbg.Warn( "WEBRTC", "streamObj is nil" )
	end

--[[
	for ln in sdp0:gmatch("(.-)\r\n") do
		_dbg.Info( "WEBRTC" , '+ "' .. ln .. '\\r\\n"' )
	end
--]]
	return rtc
end

--[[
	设置webrtc视频发送模式
		mode
			key 表示只发送关键帧
			full 表示所有的数据帧都发送
		idList
			是ID号的集合，通过 ; 做间隔
]]
function _M.SetWebRTCVideoSendMode(mode, idList)
	if mode and idList and type(idList)=="table" and #idList>0 then
		if  _M.webRtcPusher then
			for i=1, #idList do
				_M.webRtcPusher:SetWebrtcVideoSendMode( {id = idList[i], mode = mode} )
			end
			return true
		end
	end
	return false
end

--注意 一个流ID可能对应多个webrtc会话ID
function _M.UpdateStreams(id , remove)
	for streamid, sessions in pairs(_M.all_rtc_sessions) do
		if streamid == id  then -- Update me.
			local streamObj = streams.GetStreamObject( streamid )
			remove = remove or streamObj == nil
			if remove then
				for _, sid in pairs(sessions) do
					_M.RemoveWebRtcSession(sid.session)
				end
				--if streamObj then
				--	streamObj:Stop("webrtc", "stream")
				--end
				--streamObj:Start("webrtc", "stream")
			end
		end
	end
end

--[[接口说明：
		创建客户端端
	参数说明：
		id：媒体源id，可以不用填写，不填写相当于客户端只收数据，不发数据
		server_sdp：必须填写，从服务端获取到的sdp
--]]
function _M.CreateWebRTCClient(id, server_sdp)
	if  _M.webRtcPusher then
		if server_sdp then 
			_dbg.Info( "WEBRTC", "client CreateWebRTCClient server_sdp:", server_sdp )
			local streamObj = streams.GetStreamObject( id .. ":sub" )
			if not streamObj then
				_dbg.Info( "WEBRTC", "client No preview stream, so use main stream  id:", id)
				streamObj = streams.GetStreamObject( id )
			end
			local mSource
			if streamObj then
				--client
				local sess_id1, sdp2 =  _M.webRtcPusher:AddSessionClient( streamObj.syncSource:GetMediaSource(), { sdp = server_sdp })
				_dbg.Info( "WEBRTC", "client CreateWebRTCClient AddSession sess_id1:", sess_id1 )
			else 
				_dbg.Warn( "WEBRTC", "streamObj is nil" )
			end			
		else 
			_dbg.Warn( "WEBRTC", "sdp is nil" )
		end
	else 
		_dbg.Warn( "WEBRTC", "--webRtcPusher is nil" )
	end
end

function _M.GetWebRTCSdp(port)
	if  _M.webRtcPusher then
		local sdp =  _M.webRtcPusher:GetWebrtcSdp({session_id = port})
		_dbg.Info( "WEBRTC", "GetWebRTCSdp port:", port, ",sdp:", sdp )
		return sdp
	else 
		_dbg.Warn( "WEBRTC", "webRtcPusher is nil" )
	end
end
 
function _M.RemoveWebRtcSession(id)
	if  _M.webRtcPusher then
		 _M.webRtcPusher:RemoveSession(id)
	else
		_dbg.Warn( "WEBRTC", "webRtcPusher is nil" )
	end
end

function _M.DumpWebrtc()
	if  _M.webRtcPusher then
		 _M.webRtcPusher:DumpWebrtcSession()
	end
end
return _M
