local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local socket = require "socket"
local thread = require "coaf.thread"

require "kmCore"

local _M = {
    cfg_link = nil,
    save_route = "/usr/local/openresty/nginx/conf/conf.d/cameraLink.conf",
}

local function GetDNS()
	local f = io.open( "/etc/resolv.conf", "r" )
	if not f then
		return nil
	else
		local l = f:read("*l")
		local addr = "8.8.8.8"
        if l then
		    addr = l:match("^%s*nameserver%s+([%d%.]+)")
        end
		f:close()
		return addr
	end
end

local function WriteNginx(address,type)
    local address = _M.cfg_link["protocol"] .. _M.cfg_link["cameraLink"]
    local http_open = _M.cfg_link["http_open"]
    local https_open = _M.cfg_link["https_open"]
    local http_port = _M.cfg_link["http_port"]
    local https_port = _M.cfg_link["https_port"]
    local text = ""
    if https_open and http_open then
        local dns = GetDNS()
        text = [[server {
            listen       ]] .. http_port ..[[;
            listen      ]] .. https_port .. [[ ssl;
            server_name localhost;

            ssl_certificate      server.crt;
            ssl_certificate_key  server.key;

            ssl_session_cache    shared:SSL:1m;
            ssl_session_timeout  5m;

            ssl_ciphers  HIGH:!aNULL:!MD5;
            ssl_prefer_server_ciphers  on;
            location / {
                add_header Access-Control-Allow-Origin *;
                resolver ]] .. dns .. [[ 8.8.8.8 valid=10s;
                set $upstream "]] .. address .. [[";
                proxy_pass $upstream;
                    }
            }
        ]]
    
    elseif http_open then
        local dns = GetDNS()
        text = [[server {
            listen       ]] .. http_port ..[[;
            server_name localhost;
            location / {
                add_header Access-Control-Allow-Origin *;
                resolver ]] .. dns .. [[ 8.8.8.8 valid=10s;
                set $upstream "]] .. address .. [[";
                proxy_pass $upstream;
                    }
            }
        ]]
    elseif https_open then
        local dns = GetDNS()
        text = [[server {
            listen      ]] .. https_port .. [[ ssl;
            server_name localhost;

            ssl_certificate      server.crt;
            ssl_certificate_key  server.key;

            ssl_session_cache    shared:SSL:1m;
            ssl_session_timeout  5m;

            ssl_ciphers  HIGH:!aNULL:!MD5;
            ssl_prefer_server_ciphers  on;
            location / {
                add_header Access-Control-Allow-Origin *;
                resolver ]] .. dns .. [[ 8.8.8.8 valid=10s;
                set $upstream "]] .. address .. [[";
                proxy_pass $upstream;
                    }
            }
        ]]

    end
    local file = io.open(_M.save_route,"w")

    if not file then
        os.execute("/bin/mkdir -p /usr/local/openresty/nginx/conf/conf.d")
        file = io.open(_M.save_route,"w")
    end


    if file then
        file:write(text)
        file:close()
        local a = os.execute("/usr/local/openresty/nginx/sbin/nginx -t")
        if a then
            os.execute("/usr/local/openresty/nginx/sbin/nginx -s reload")
        else
            return false,"nginx set error"
        end
    else
        return false,"open file error"
    end
end




function _M.GetLink()
    return _M.cfg_link
end
function _M.SetLink(src,protocol,type)
    _M.cfg_link["cameraLink"] = src
    _M.cfg_link["protocol"] =protocol
    _M.cfg_link:__TRIGGER(true)

    if not src or not protocol then
        return false
    end
    WriteNginx()
    return true
end

function _M.SetWebPort(cameraLinkPort,cameraLinksPort,http_open,https_open)
    _M.cfg_link["http_port"] = cameraLinkPort
    _M.cfg_link["https_port"] = cameraLinksPort
    _M.cfg_link["http_open"] = http_open
    _M.cfg_link["https_open"] = https_open
    _M.cfg_link:__TRIGGER(true)
    WriteNginx()
    return true
end




function _M.Init()
	local cfg_link = require "cameraLink_config"
	_M.cfg_link = cfg_link.Load()
	if not _M.cfg_link then
		_dbg.Error( "CAMERALINK", "Fail to load cameraLink configure file!" )
		_M.cfg_link = {
			__TRIGGER = function() end
		}
	end
end

return _M
