local coaf = require 'coaf.core'
local _dbg = require 'coaf.debug'
local client = require 'coaf.client'
local CO = require 'mod_coservice'
local pt = require 'pl.pretty'
require "kmCore"

local _M = {
	config = nil,
    rtmpServer = nil
}

---------------------------------------------

function _M.Init()
	_M.config = require "rtmp_service_config".Load()

	pt.dump(_M.config )
	
	local _port = _M.config.server_port or 1935
	_M.rtmpServer = KM_STREAMING.RTMP.SERVER.createNew("RtmpServer")
	if _M.rtmpServer then
		_M.rtmpServer :Start( {port = _port} ) --参数为port
	else
		_dbg.Error("RTMP_SERVICE", "CreateNew rtmp server object error!")
	end
end

function _M.Destroy()
	_M.config = nil
	if _M.rtmpServer then
		_M.rtmpServer:UnInit()
	end
	_M.rtmpServer = nil
end

function _M.AddSession(opts)
	if not opts then
		return false
	end

	if _M.rtmpServer then
		_M.rtmpServer:AddRtmpUrl( opts )
	end
	return true
end

function _M.RemoveSession(session)
	if _M.rtmpServer and session then
		_M.rtmpServer:RemoveRtmpUrl( {url = session} )
	end
	return true
end

function _M.GetSessionStatus(session)
	local status = nil
	if _M.rtmpServer and session then
		status = _M.rtmpServer:FindRtmpStatus( {url = session} )
	end
	return status
end

function _M.UpdateServerPort(_port)
	if _M.rtmpServer and _port then
		 _M.rtmpServer:UpdateRtmpPort( {port = _port} )
	end

	if _M.config.server_port ~= _port and _port then
		_M.config.server_port = _port
		_M.config:__TRIGGER(true)
	 end

	return true
end

function _M.GetServerPort()
	 return (_M.config.server_port  or 1935)
end

return _M
