#!/usr/bin/env lua
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local CO = require "mod_coservice"
local pt = require "pl.pretty"

local mo_RtmpService = require "mod_RtmpService"


local withLog = false
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /var/log/" )
	local logF = roll.Create("/var/log/rtmp_service.log")
	if not logF or not logF:init() then
		_dbg.Error("RTMP_SERVICE", "Fail to open log file: /var/log/rtmp_service.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

--------------------------------------

coaf.Init( "rtmp_service" )

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "RTMPSERVER" )
	stdio.Start()
end

service.InitSocket()
client.InitSocket( socket, adapter )

local cosvc_rtmp

local function Main(args)

	mo_RtmpService.Init()

    cosvc_rtmp = service.New( "rtmp_service", "unix:/var/run/rtmp_service.sock" )
	if cosvc_rtmp then
		_dbg.Info( "RTMP_SERVICE", "CoService of 'rtmp_service' is created." )
		cosvc_rtmp:CreateCoServerProxy()
		cosvc_rtmp:ExposeObject("/rtmp_service", mo_RtmpService )
	else
		_dbg.Error( "RTMP_SERVICE", "Fail created CoService!" )
	end
end

coaf.Start(Main, arg)
mo_RtmpService.Destroy()
_dbg.Print( "RTMP_SERVICE", "Service terminated." )

