--The Sony Visca protocol

local function RestrictAddr(addr)
	if not addr or addr <= 0 then
		return 1
	elseif addr > 0x0f then
		return 0x0f
	else
		return addr
	end
end

local _M = {

name = "Sony Visca-aida",

AckFilter = nil,	--TODO: Process the ack messages, but now ignore it.
maxAddresses = 8,

supports = {
    ["ExposureMode"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x04,0x39,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device == 0 or device == 3  then
                return {{0x80+RestrictAddr(addr),0x01,0x04,0x39,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,

    ["DCIris"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x7e,0x04,0x55,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device == 0 or device == 1 or device == 2  then
                return {{0x80+RestrictAddr(addr),0x01,0x7e,0x04,0x55,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,

    ["AntiFlicker"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x7e,0x04,0x53,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device == 0 or device == 2 or device == 3  then
                return {{0x80+RestrictAddr(addr),0x01,0x7e,0x04,0x53,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,

    ["Gain"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x04,0x4c,0xff}}
        elseif attribute == "reset" then
            return {{0x80+RestrictAddr(addr),0x01,0x04,0x0c,0x00,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=15 then
                return {{0x80+RestrictAddr(addr),0x01,0x04,0x4c,0x00,0x00,0x00,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,

    ["shutter"] = function(addr,attribute) -- 1和0一样，5和6一样
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x04,0x4a,0xff}}
        elseif attribute == "reset" then
            return {{0x80+RestrictAddr(addr),0x01,0x04,0x0a,0x00,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=21 then
                local ones_place = device%16 -- 个位
                local tens_place = device//16 -- 十位
                return {{0x80+RestrictAddr(addr),0x01,0x04,0x4A,0x00,0x00,0x00+tens_place,0x00+ones_place,0xFF}}
            end
            return false
        end
        return false
    end,

    ["WhiteBalanceMode"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x04,0x35,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=5 then
                return {{0x80+RestrictAddr(addr),0x01,0x04,0x35,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,

    ["RedGain"] = function(addr,attribute)
        if attribute == "get" then
            return {0x80+RestrictAddr(addr),0x09,0x04,0x43,0xff}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=255 then
                local ones_place = device%16 -- 个位
                local tens_place = device//16 -- 十位
                return {{0x80+RestrictAddr(addr),0x01,0x04,0x43,0x00,0x00,0x00+tens_place,0x00+ones_place,0xff}}
            end
            return false
        end
        return false
    end,

    ["BlueGain"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x04,0x44,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=255 then
                local ones_place = device%16 -- 个位
                local tens_place = device//16 -- 十位
                return {{0x80+RestrictAddr(addr),0x01,0x04,0x44,0x00,0x00,0x00+tens_place,0x00+ones_place,0xff}}
            end
            return false
        end
        return false
    end,

    ["BakLightCompensation"] = function(addr,attribute)
        if attribute == "get" then
            return {0x80+RestrictAddr(addr),0x09,0x04,0x33,0xff}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device == 2 or device == 3 then
                return {{0x80+RestrictAddr(addr),0x01,0x04,0x33,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,

    ["Gamma"] = function(addr,attribute) -- 15=default
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x04,0x5b,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=21 then
                return {{0x80+RestrictAddr(addr),0x01,0x04,0x5b,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,

    ["Brightness"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x7e,0x04,0x52,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=20 then
                return {{0x80+RestrictAddr(addr),0x01,0x7e,0x04,0x52,0x00,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,

    ["Sharpness"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x04,0x42,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=20 then
                local ones_place = device%16 -- 个位
                local tens_place = device//16 -- 十位
                return {{0x80+RestrictAddr(addr),0x01,0x04,0x42,0x00,0x00,0x00+tens_place,0x00+ones_place,0xff}}
            end
            return false
        end
        return false
    end,

    ["Contrast"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x7e,0x04,0x51,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=20 then
                local ones_place = device%16 -- 个位
                local tens_place = device//16 -- 十位
                return {{0x80+RestrictAddr(addr),0x01,0x7e,0x04,0x51,0x00+tens_place,0x00+ones_place,0xff}}
            end
            return false
        end
        return false
    end,

    ["Saturation"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x04,0x49,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=14 then
                return {{0x80+RestrictAddr(addr),0x01,0x04,0x49,0x00,0x00,0x00,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,

    ["NR"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x04,0x53,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=3 then
                return {{0x80+RestrictAddr(addr),0x01,0x04,0x53,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,
    ["Resolution"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x04,0x24,0x72,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            local ones_place = device%16 -- 个位
            local tens_place = device//16 -- 十位
            return {{0x80+RestrictAddr(addr),0x01,0x04,0x24,0x72,0x00+tens_place,0x00+ones_place,0xff}}
        end
        return false
    end,
    ["lens-control"] = function(addr,attribute)
        return {{0x80+RestrictAddr(addr),0x09,0x7e,0x7e,0x00,0xff}}
    end,
    ["camera-control"] = function(addr,attribute)
        return {{0x80+RestrictAddr(addr),0x09,0x7e,0x7e,0x01,0xff}}
    end,
    ["other-control"] = function(addr,attribute)
        return {{0x80+RestrictAddr(addr),0x09,0x7e,0x7e,0x02,0xff}}
    end,
    ["extended-function1"] = function(addr,attribute)
        return {{0x80+RestrictAddr(addr),0x09,0x7e,0x7e,0x03,0xff}}
    end,
    ["extended-function2"] = function(addr,attribute)
        return {{0x80+RestrictAddr(addr),0x09,0x7e,0x7e,0x04,0xff}}
    end,
    ["extended-function3"] = function(addr,attribute)
        return {{0x80+RestrictAddr(addr),0x09,0x7e,0x7e,0x05,0xff}}
    end,
    ["audioMode"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x7e,0x05,0x11,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=1 and device <=3 then
                return {{0x80+RestrictAddr(addr),0x01,0x7e,0x05,0x11,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,
    ["audioSampleBits"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x7e,0x05,0x12,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=3 then
                return {{0x80+RestrictAddr(addr),0x01,0x7e,0x05,0x12,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,
    ["audioVolume"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x7e,0x05,0x43,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=12 then
                return {{0x80+RestrictAddr(addr),0x01,0x7e,0x05,0x43,0x00,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,
    ["audioAttenuation"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x7e,0x05,0x14,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=2 and device <=3 then
                return {{0x80+RestrictAddr(addr),0x01,0x7e,0x05,0x14,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,
    ["audioAgcLimit"] = function(addr,attribute)
        if attribute == "get" then
            return {{0x80+RestrictAddr(addr),0x09,0x7e,0x05,0x46,0xff}}
        elseif type(tonumber(attribute)) == "number" then
            local device = tonumber(attribute)
            if device>=0 and device <=12 then
                return {{0x80+RestrictAddr(addr),0x01,0x7e,0x05,0x46,0x00,0x00+device,0xff}}
            end
            return false
        end
        return false
    end,
}
}

return _M