#!/bin/sh

export PROD_ID="U40"
export TARGET_TMP_DIR="/upgrade/.upgrade_tmp"

export SWVER="1.86.0200"
export LUA_VERSION="5.3"

export ROOT_REQ_KB="12292"
export DATA_REQ_KB="3196"
export UPGRADE_RETURNS=1

#!/bin/sh
export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
mkdir -p /data/update_log
mkdir -p $TARGET_TMP_DIR/_UPGRADE

updatetime=`date "+%Y-%m-%d %H:%M:%S"`
updatetimecode=`date "+%Y%m%d%H%M"`
WITH_WARNING=0
WITH_ERROR=0
logfile="/data/update_log/$SWVER-$updatetimecode.log"

echo "Upgrade to Software version $SWVER..." > $logfile
echo "Upgrade time: $updatetime" >> $logfile

#NOTE: Must disable watch-dog while upgrading!
if [ -x /sbin/wdog ]; then
	/sbin/wdog disable
	echo "Disabled watch-dog while upgrading." >> $logfile
fi

echo "------------------------------------" >> $logfile

if [ $# -gt 0 ]; then
    cd $1
else
    cd /upgrade
fi

tar xzf ./patch.tar.gz -C $TARGET_TMP_DIR/_UPGRADE 
if [ $? -ne 0 ]; then
	echo "Upgrading fail! Error to extract the patch file!" >> $logfile
	rm -Rf $TARGET_TMP_DIR
	exit 1
fi

rm -Rf ./patch.tar.gz
cd $TARGET_TMP_DIR	#Change to target position

getDiskFree()
{
	#Because the K2E busybox system only support human format
	local freeSpaces=`df -h $1 | sed 1d | awk '{print $4}'`
	if [ "${freeSpaces/G/}" != "${freeSpaces}" ]; then
		#Is GB
		freeSpaces=${freeSpaces/G/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces * 1024 * 1024))"`
	elif [ "${freeSpaces/M/}" != "${freeSpaces}" ]; then
		#Is MB
		freeSpaces=${freeSpaces/M/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces * 1024))"`
	elif [ "${freeSpaces/K/}" != "${freeSpaces}" ]; then
		#Is KB
		freeSpaces=${freeSpaces/K/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces))"`
	fi
	echo $freeSpaces
	exit 0
}

root_free_sz=`getDiskFree /`
data_free_sz=`getDiskFree /data`

root_free_sz=`expr $root_free_sz - 1024`	#Keep 1MB at least
data_free_sz=`expr $data_free_sz - 1024`	#Keep 1MB at least

echo "ROOT: Require $ROOT_REQ_KB KB, valid $root_free_sz KB." >> $logfile
echo "DATA: Require $DATA_REQ_KB KB, valid $data_free_sz KB." >> $logfile

if [ $root_free_sz -lt $ROOT_REQ_KB -o $data_free_sz -lt $DATA_REQ_KB ]; then
	echo "ERROR: Insufficent disk spaces" >> $logfile
	exit 1
fi

upgrade()
{
	if [ -d "$TARGET_TMP_DIR/_UPGRADE/fakeroot" ]; then
		cp -Rf "$TARGET_TMP_DIR/_UPGRADE/fakeroot/"* / >> $logfile 2>&1
	fi

	if [ -d "$TARGET_TMP_DIR/_UPGRADE/datafs" ]; then
		if [ ! -d /data ]; then
			mkdir /data
		fi
		cp -Rf "$TARGET_TMP_DIR/_UPGRADE/datafs/"* /data/ >> $logfile 2>&1
	fi

	if [ -d "$TARGET_TMP_DIR/_UPGRADE/boot" ]; then
		this_part_id="OTHER"
		default_boot_for_part_id="OTHER"

		if [ -f /boot/PART_ID ]; then
			this_part_id=`cat /boot/PART_ID`
		fi
		if [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/boot.def" ]; then
			default_boot_for_part_id=`cat $TARGET_TMP_DIR/_UPGRADE/boot/boot.def`
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/boot.def" 
		fi

		#Don't touch PART_ID, if exists.
		#rm "$TARGET_TMP_DIR/_UPGRADE/boot/PART_ID"

		#2 cases: 1) part_id matched; 2) both part_id and 'boot.def' are empty ("OTHER" identified).
		if [ "$default_boot_for_part_id" = "$this_part_id" ]; then
			#Keep default BOOT.BIN, drop others.
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"BOOT-*.BIN
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit-*.bin
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit-*.bin
		elif [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT-$this_part_id.BIN" ]; then
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT.BIN"
			mv "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT-$this_part_id.BIN" "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT.BIN"
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"BOOT-*.BIN

			if [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/enc_bit-$this_part_id.bin" ]; then
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit.bin
				mv "$TARGET_TMP_DIR/_UPGRADE/boot/enc_bit-$this_part_id.bin" "$TARGET_TMP_DIR/_UPGRADE/boot/enc_bit.bin"
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit-*.bin
			fi

			if [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/dec_bit-$this_part_id.bin" ]; then
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit.bin
				mv "$TARGET_TMP_DIR/_UPGRADE/boot/dec_bit-$this_part_id.bin" "$TARGET_TMP_DIR/_UPGRADE/boot/dec_bit.bin"
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit-*.bin
			fi
		else
			#Don't touch BOOT.BIN and other .bit files.
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"BOOT*.BIN
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit*.bin
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit*.bin
		fi
		cp -Rf "$TARGET_TMP_DIR/_UPGRADE/boot/"* /boot/ >> $logfile 2>&1
	fi

	return 0
}

drop()
{
	if [ "$1" = "fakeroot" ]; then
		if [ -e "$2" ]; then
			rm -Rf "$2"
			if [ $? -ne 0 ]; then
			    echo "WARNING: FAIL to drop un-used item [$2]." >> $logfile
			    WITH_WARNING=`expr $WITH_WARNING + 1`
			    return 1
			fi
		fi
	elif [ "$1" = "datafs" ]; then
		if [ -e "/data$2" ]; then
			rm -Rf "/data$2"
			if [ $? -ne 0 ]; then
			    echo "WARNING: FAIL to drop un-used item [/data$2]." >> $logfile
			    WITH_WARNING=`expr $WITH_WARNING + 1`
			    return 1
			fi
		fi
	fi
    return 0
}

## The upgrading content below: (generated by release tool)

drop datafs "/web/html/static/css/app.403344fb9931fe337946b6adf853d536.css"
drop datafs "/web/html/static/css/app.55d73c5a12f0e229b0d6416cc0a02a15.css"
drop datafs "/web/html/static/css/app.6b4126cfc19cf6e3a17f4dae46c4c8dd.css"
drop datafs "/web/html/static/css/app.a428b37267c28b0faa0e9e81aa2dcb6f.css"
drop datafs "/web/html/static/css/app.a78e073e4363d91b51b25d3a5329505b.css"
drop datafs "/web/html/static/css/app.ab4fe621574a922413a9579cd38f1d46.css"
drop datafs "/web/html/static/css/app.c5067c67db82a5e777a9a5b708d36eca.css"
drop datafs "/web/html/static/css/app.cd22041bbeb2b113ea19f72d624223cb.css"
drop datafs "/web/html/static/css/app.d2b4465ae79a32055766df19abd242cb.css"
drop datafs "/web/html/static/css/app.d8e2717647fd7e44c8cd85e9f20b66db.css"
drop datafs "/web/html/static/fonts/iconfont.456b7c2.eot"
drop datafs "/web/html/static/fonts/iconfont.4c0cf5b.eot"
drop datafs "/web/html/static/fonts/iconfont.e45de52.ttf"
drop datafs "/web/html/static/fonts/iconfont.f4fb007.ttf"
drop datafs "/web/html/static/img/1.381ae97.png"
drop datafs "/web/html/static/img/3.1b7dc9b.png"
drop datafs "/web/html/static/img/4.b6b8a88.png"
drop datafs "/web/html/static/img/5.f0d630c.png"
drop datafs "/web/html/static/img/6.a619365.png"
drop datafs "/web/html/static/img/bg.44d2975.jpg"
drop datafs "/web/html/static/img/iconfont.1760417.svg"
drop datafs "/web/html/static/img/iconfont.fe1f0a9.svg"
drop datafs "/web/html/static/img/logo.*.png"
drop datafs "/web/html/static/js/app.038c38b3987c6aab8a73.js"
drop datafs "/web/html/static/js/app.12058e54bb2bb57e0e42.js"
drop datafs "/web/html/static/js/app.47b2a5d5d7470532b5fe.js"
drop datafs "/web/html/static/js/app.58823bda59f9a2d4be39.js"
drop datafs "/web/html/static/js/app.603f8f7a88681370834a.js"
drop datafs "/web/html/static/js/app.6c5640867f167d8a7903.js"
drop datafs "/web/html/static/js/app.81cd8d451738da5e131e.js"
drop datafs "/web/html/static/js/app.a086bafee1d69ad73b40.js"
drop datafs "/web/html/static/js/app.c76e02c14883e2547a19.js"
drop datafs "/web/html/static/js/app.e73ecf0fc2b0e3ac0833.js"
drop datafs "/web/html/static/js/decoderMode.01e1359ccd118c1fed3d.js"
drop datafs "/web/html/static/js/decoderMode.044217fc4280a976491c.js"
drop datafs "/web/html/static/js/decoderMode.0af4b38727e6c938ffe1.js"
drop datafs "/web/html/static/js/decoderMode.6231cedeec00a880b2ad.js"
drop datafs "/web/html/static/js/decoderMode.732370b1d691f3a24582.js"
drop datafs "/web/html/static/js/decoderMode.976af6d8c6c28642089a.js"
drop datafs "/web/html/static/js/decoderMode.be92c68b7c7057713eed.js"
drop datafs "/web/html/static/js/decoderMode.fddc513be8ad61541aba.js"
drop datafs "/web/html/static/js/encoderMode.40117591a8bf93fb8448.js"
drop datafs "/web/html/static/js/encoderMode.59c5a02067f5966aa6d2.js"
drop datafs "/web/html/static/js/encoderMode.940667e911ae628db9ea.js"
drop datafs "/web/html/static/js/encoderMode.b2025aac19934eb88221.js"
drop datafs "/web/html/static/js/encoderMode.d3d9a97b94871ed22222.js"
drop datafs "/web/html/static/js/firmwareudp.3e240eda5a272d89c3e0.js"
drop datafs "/web/html/static/js/firmwareudp.d1abe279ddcf8b1417c1.js"
drop datafs "/web/html/static/js/firmwareudp.deec2552a51ce8502d39.js"
drop datafs "/web/html/static/js/home.1c6985293d16903dec6e.js"
drop datafs "/web/html/static/js/home.5cb63ba751e0badca11d.js"
drop datafs "/web/html/static/js/home.74b81a7f366acba3b1c5.js"
drop datafs "/web/html/static/js/home.a8ae5d20cb56ec52579a.js"
drop datafs "/web/html/static/js/home.c9095dfe8c52850bcd90.js"
drop datafs "/web/html/static/js/home.ebc7e22abe27865c8741.js"
drop datafs "/web/html/static/js/home.ec3295596571279a30e2.js"
drop datafs "/web/html/static/js/manifest.2f8e6f5c3719537062c0.js"
drop datafs "/web/html/static/js/manifest.674bf4162335d4fa51b1.js"
drop datafs "/web/html/static/js/manifest.697da8bf906a1a862f68.js"
drop datafs "/web/html/static/js/manifest.8281b1b12e50db8f51e3.js"
drop datafs "/web/html/static/js/manifest.9f98a2dcabb444fb108f.js"
drop datafs "/web/html/static/js/manifest.a324a7b56b5c703c44c7.js"
drop datafs "/web/html/static/js/manifest.ae1a46d63a0e9f8b14d1.js"
drop datafs "/web/html/static/js/manifest.b702f3039687518642b5.js"
drop datafs "/web/html/static/js/manifest.c8f7b2007719a8a108b3.js"
drop datafs "/web/html/static/js/manifest.e1a41d41c0894cb399a2.js"
drop datafs "/web/html/static/js/networksetup.399ed63084cd41d5f193.js"
drop datafs "/web/html/static/js/networksetup.92b4b8a62b3af840e84b.js"
drop datafs "/web/html/static/js/networksetup.ba63bde5d90a48004f86.js"
drop datafs "/web/html/static/js/Newtek.25c96b7b5eae9afd159f.js"
drop datafs "/web/html/static/js/OpenSourceComponents.02064f797f1cb00d3e6b.js"
drop datafs "/web/html/static/js/softwareProtocol.17dbe7065930cedc86d9.js"
drop datafs "/web/html/static/js/softwareProtocol.964ebf68d9c71998b9d9.js"
drop datafs "/web/html/static/js/softwareProtocol.f67e853e2b9c343f7406.js"
drop datafs "/web/html/static/js/superMode.016d00d0f394ab2fdae6.js"
drop datafs "/web/html/static/js/superMode.a0bd5425e7d24f81114c.js"
drop datafs "/web/html/static/js/system-time.0183a96e538e4b9bf844.js"
drop datafs "/web/html/static/js/system-time.6a27ca09e27730f88be6.js"
drop datafs "/web/html/static/js/users.22f45e9168c0bc2b15fe.js"
drop datafs "/web/html/static/js/users.8d6a2fb4643855ca3812.js"
drop datafs "/web/html/static/js/users.e878b56d24269a49ac6e.js"
drop datafs "/web/html/static/js/users.f999d4c67dc1e954dcf0.js"
drop datafs "/web/html/static/js/vendor.161f9f7535623ba13d16.js"
drop datafs "/web/html/static/js/vendor.81e453a011bbeb9a63c3.js"
drop datafs "/web/html/static/js/vendor.d18b4a2ecef6ffe3f07d.js"
drop fakeroot "/etc/coserver/ANY/ANY"
drop fakeroot "/etc/coserver/local/hdmi_splitter"
drop fakeroot "/.stage.apt/var/cache/apt/archives/lock"
drop fakeroot "/.stage.apt/var/lib/apt/lists/lock"
drop fakeroot "/.stage.apt/var/log/apt/term.log"
drop fakeroot "/.targetroot.apt/var/cache/apt/archives/lock"
drop fakeroot "/.targetroot.apt/var/lib/apt/lists/lock"
drop fakeroot "/.targetroot.apt/var/log/apt/term.log"
drop fakeroot "/usr/lib/libndi.so.4.0.0"
drop fakeroot "/usr/lib/libndi.so.4.1.0"
drop fakeroot "/usr/lib/libndi.so.4.1.4"
drop fakeroot "/usr/lib/libndi.so.4.2.0"
drop fakeroot "/usr/lib/libndi.so.4.5.0"
drop fakeroot "/usr/lib/lua/5.3/it6663drv.so"
drop fakeroot "/usr/lib/lua/5.3/sii9136drv.so"
drop fakeroot "/usr/local/bin/codec/chips/ite66021.lua"
drop fakeroot "/usr/local/bin/codec/chips/ite6661.lua"
drop fakeroot "/usr/local/bin/codec/chips/sii9136.lua"
drop fakeroot "/usr/local/bin/codec/hdmi_chips.lua"
drop fakeroot "/usr/local/bin/codec/iocfg.lua"
drop fakeroot "/usr/local/bin/hdmi_splitter.lua"
drop fakeroot "/usr/sbin/usb_modeswitch"
drop fakeroot "/usr/sbin/usb_modeswitch_dispatcher"
drop fakeroot "/var/lib/dpkg/lock"
drop fakeroot "/var/lib/dpkg/triggers/Lock"
upgrade
#!/bin/sh

#Drop EULA accept status
rm -Rf /data/configs/EULA/* > /dev/null 2>&1
## Upgrading done at here.

#Drop EULA
rm -Rf /data/configs/EULA/* > /dev/null 2>&1

#Update version.
sed -i 's/SOFTWARE_VERSION=.*$/SOFTWARE_VERSION=[['$SWVER']]/g' /usr/share/lua/$LUA_VERSION/version.lua >> $logfile 2>&1
sed -i 's/SOFTWARE_VERSION=.*$/SOFTWARE_VERSION=[['$SWVER']]/g' /usr/local/openresty/lualib/version.lua >> $logfile 2>&1

echo "------------------------------------" >> $logfile
echo "Finished upgrading. [$WITH_ERROR errors, $WITH_WARNING warnnings.]" >> $logfile

rm -Rf $TARGET_TMP_DIR/_UPGRADE
sync
sync
exit $UPGRADE_RETURNS

